# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

import munch
from requests import exceptions as _rex

from shade import _log


class OpenStackCloudException(Exception):

    log_inner_exceptions = False

    def __init__(self, message, extra_data=None, **kwargs):
        args = [message]
        if extra_data:
            if isinstance(extra_data, munch.Munch):
                extra_data = extra_data.toDict()
            args.append("Extra: {0}".format(str(extra_data)))
        super(OpenStackCloudException, self).__init__(*args, **kwargs)
        self.extra_data = extra_data
        self.inner_exception = sys.exc_info()
        self.orig_message = message

    def log_error(self, logger=None):
        if not logger:
            logger = _log.setup_logging(__name__)
        if self.inner_exception and self.inner_exception[1]:
            logger.error(self.orig_message, exc_info=self.inner_exception)

    def __str__(self):
        message = Exception.__str__(self)
        if (self.inner_exception and self.inner_exception[1]
                and not self.orig_message.endswith(
                    str(self.inner_exception[1]))):
            message = "%s (Inner Exception: %s)" % (
                message,
                str(self.inner_exception[1]))
        if self.log_inner_exceptions:
            self.log_error()
        return message


class OpenStackCloudTimeout(OpenStackCloudException):
    pass


class OpenStackCloudUnavailableExtension(OpenStackCloudException):
    pass


class OpenStackCloudUnavailableFeature(OpenStackCloudException):
    pass


class OpenStackCloudHTTPError(OpenStackCloudException, _rex.HTTPError):

    def __init__(self, *args, **kwargs):
        OpenStackCloudException.__init__(self, *args, **kwargs)
        _rex.HTTPError.__init__(self, *args, **kwargs)


class OpenStackCloudURINotFound(OpenStackCloudHTTPError):
    pass

# Backwards compat
OpenStackCloudResourceNotFound = OpenStackCloudURINotFound


# Logic shamelessly stolen from requests
def raise_from_response(response):
    msg = ''
    if 400 <= response.status_code < 500:
        msg = '({code}) Client Error: {reason} for url: {url}'.format(
            code=response.status_code,
            reason=response.reason,
            url=response.url)
    elif 500 <= response.status_code < 600:
        msg = '({code}) Server Error: {reason} for url: {url}'.format(
            code=response.status_code,
            reason=response.reason,
            url=response.url)

    # Special case 404 since we raised a specific one for neutron exceptions
    # before
    if response.status_code == 404:
        raise OpenStackCloudURINotFound(msg, response=response)
    if msg:
        raise OpenStackCloudHTTPError(msg, response=response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock
import novaclient.exceptions as nova_exceptions

import shade
from shade.tests.unit import base
from shade.tests import fakes


class TestServerConsole(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_get_server_console_dict(self, mock_nova):
        server = dict(id='12345')
        self.cloud.get_server_console(server)

        mock_nova.servers.list.assert_not_called()
        mock_nova.servers.get_console_output.assert_called_once_with(
            server='12345', length=None)

    @mock.patch.object(shade.OpenStackCloud, 'has_service')
    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_get_server_console_name_or_id(self, mock_nova, mock_has_service):
        server = '12345'

        fake_server = fakes.FakeServer(server, '', 'ACTIVE')
        mock_nova.servers.get.return_value = fake_server
        mock_nova.servers.list.return_value = [fake_server]
        mock_has_service.return_value = False

        self.cloud.get_server_console(server)

        mock_nova.servers.get_console_output.assert_called_once_with(
            server='12345', length=None)

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_get_server_console_no_console(self, mock_nova):
        server = dict(id='12345')
        exc = nova_exceptions.BadRequest(
            'There is no such action: os-getConsoleOutput')
        mock_nova.servers.get_console_output.side_effect = exc
        log = self.cloud.get_server_console(server)

        self.assertEqual('', log)
        mock_nova.servers.list.assert_not_called()
        mock_nova.servers.get_console_output.assert_called_once_with(
            server='12345', length=None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock
import os_client_config

from os_client_config import exceptions as occ_exc

from shade import exc
from shade import inventory
from shade import meta
from shade.tests import fakes
from shade.tests.unit import base


class TestInventory(base.TestCase):

    def setUp(self):
        super(TestInventory, self).setUp()

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test__init(self, mock_cloud, mock_config):
        mock_config.return_value.get_all_clouds.return_value = [{}]

        inv = inventory.OpenStackInventory()

        mock_config.assert_called_once_with(
            config_files=os_client_config.config.CONFIG_FILES
        )
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        self.assertTrue(mock_config.return_value.get_all_clouds.called)

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test__init_one_cloud(self, mock_cloud, mock_config):
        mock_config.return_value.get_one_cloud.return_value = [{}]

        inv = inventory.OpenStackInventory(cloud='supercloud')

        mock_config.assert_called_once_with(
            config_files=os_client_config.config.CONFIG_FILES
        )
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        self.assertFalse(mock_config.return_value.get_all_clouds.called)
        mock_config.return_value.get_one_cloud.assert_called_once_with(
            'supercloud')

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test__raise_exception_on_no_cloud(self, mock_cloud, mock_config):
        """
        Test that when os-client-config can't find a named cloud, a
        shade exception is emitted.
        """
        mock_config.return_value.get_one_cloud.side_effect = (
            occ_exc.OpenStackConfigException()
        )
        self.assertRaises(exc.OpenStackCloudException,
                          inventory.OpenStackInventory,
                          cloud='supercloud')
        mock_config.return_value.get_one_cloud.assert_called_once_with(
            'supercloud')

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test_list_hosts(self, mock_cloud, mock_config):
        mock_config.return_value.get_all_clouds.return_value = [{}]

        inv = inventory.OpenStackInventory()

        server = dict(id='server_id', name='server_name')
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        inv.clouds[0].list_servers.return_value = [server]
        inv.clouds[0].get_openstack_vars.return_value = server

        ret = inv.list_hosts()

        inv.clouds[0].list_servers.assert_called_once_with(detailed=True)
        self.assertFalse(inv.clouds[0].get_openstack_vars.called)
        self.assertEqual([server], ret)

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test_list_hosts_no_detail(self, mock_cloud, mock_config):
        mock_config.return_value.get_all_clouds.return_value = [{}]

        inv = inventory.OpenStackInventory()

        server = self.cloud._normalize_server(
            meta.obj_to_dict(fakes.FakeServer(
                '1234', 'test', 'ACTIVE', addresses={})))
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        inv.clouds[0].list_servers.return_value = [server]

        inv.list_hosts(expand=False)

        inv.clouds[0].list_servers.assert_called_once_with(detailed=False)
        self.assertFalse(inv.clouds[0].get_openstack_vars.called)

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test_search_hosts(self, mock_cloud, mock_config):
        mock_config.return_value.get_all_clouds.return_value = [{}]

        inv = inventory.OpenStackInventory()

        server = dict(id='server_id', name='server_name')
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        inv.clouds[0].list_servers.return_value = [server]
        inv.clouds[0].get_openstack_vars.return_value = server

        ret = inv.search_hosts('server_id')
        self.assertEqual([server], ret)

    @mock.patch("os_client_config.config.OpenStackConfig")
    @mock.patch("shade.OpenStackCloud")
    def test_get_host(self, mock_cloud, mock_config):
        mock_config.return_value.get_all_clouds.return_value = [{}]

        inv = inventory.OpenStackInventory()

        server = dict(id='server_id', name='server_name')
        self.assertIsInstance(inv.clouds, list)
        self.assertEqual(1, len(inv.clouds))
        inv.clouds[0].list_servers.return_value = [server]
        inv.clouds[0].get_openstack_vars.return_value = server

        ret = inv.get_host('server_id')
        self.assertEqual(server, ret)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_compute
----------------------------------

Functional tests for `shade` compute methods.
"""

import six

from shade import exc
from shade.tests.functional import base
from shade.tests.functional.util import pick_flavor, pick_image
from shade import _utils


class TestCompute(base.BaseFunctionalTestCase):
    def setUp(self):
        super(TestCompute, self).setUp()
        self.flavor = pick_flavor(self.demo_cloud.list_flavors())
        if self.flavor is None:
            self.assertFalse('no sensible flavor available')
        self.image = pick_image(self.demo_cloud.list_images())
        if self.image is None:
            self.assertFalse('no sensible image available')
        self.server_name = self.getUniqueString()

    def _cleanup_servers_and_volumes(self, server_name):
        """Delete the named server and any attached volumes.

        Adding separate cleanup calls for servers and volumes can be tricky
        since they need to be done in the proper order. And sometimes deleting
        a server can start the process of deleting a volume if it is booted
        from that volume. This encapsulates that logic.
        """
        server = self.demo_cloud.get_server(server_name)
        if not server:
            return
        volumes = self.demo_cloud.get_volumes(server)
        self.demo_cloud.delete_server(server.name, wait=True)
        for volume in volumes:
            if volume.status != 'deleting':
                self.demo_cloud.delete_volume(volume.id, wait=True)

    def test_create_and_delete_server(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.id, server['flavor']['id'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))

    def test_create_server_image_flavor_dict(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image={'id': self.image.id},
            flavor={'id': self.flavor.id},
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.id, server['flavor']['id'])
        self.assertIsNotNone(server['adminPass'])
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))

    def test_get_server_console(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        for _ in _utils._iterate_timeout(
                5, "Did not get more than 0 lines in the console log"):
            log = self.demo_cloud.get_server_console(server=server)
            self.assertTrue(isinstance(log, six.string_types))
            if len(log) > 0:
                break

    def test_get_server_console_name_or_id(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        for _ in _utils._iterate_timeout(
                5, "Did not get more than 0 lines in the console log"):
            log = self.demo_cloud.get_server_console(server=self.server_name)
            self.assertTrue(isinstance(log, six.string_types))
            if len(log) > 0:
                break

    def test_get_server_console_bad_server(self):
        self.assertRaises(
            exc.OpenStackCloudException,
            self.demo_cloud.get_server_console,
            server=self.server_name)

    def test_create_and_delete_server_with_admin_pass(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            admin_pass='sheiqu9loegahSh',
            wait=True)
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(self.image.id, server['image']['id'])
        self.assertEqual(self.flavor.id, server['flavor']['id'])
        self.assertEqual(server['adminPass'], 'sheiqu9loegahSh')
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))

    def test_get_image_id(self):
        self.assertEqual(
            self.image.id, self.demo_cloud.get_image_id(self.image.id))
        self.assertEqual(
            self.image.id, self.demo_cloud.get_image_id(self.image.name))

    def test_get_image_name(self):
        self.assertEqual(
            self.image.name, self.demo_cloud.get_image_name(self.image.id))
        self.assertEqual(
            self.image.name, self.demo_cloud.get_image_name(self.image.name))

    def _assert_volume_attach(self, server, volume_id=None, image=''):
        self.assertEqual(self.server_name, server['name'])
        self.assertEqual(image, server['image'])
        self.assertEqual(self.flavor.id, server['flavor']['id'])
        volumes = self.demo_cloud.get_volumes(server)
        self.assertEqual(1, len(volumes))
        volume = volumes[0]
        if volume_id:
            self.assertEqual(volume_id, volume['id'])
        else:
            volume_id = volume['id']
        self.assertEqual(1, len(volume['attachments']), 1)
        self.assertEqual(server['id'], volume['attachments'][0]['server_id'])
        return volume_id

    def test_create_boot_from_volume_image(self):
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            boot_from_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server)
        volume = self.demo_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertEqual(True, volume['bootable'])
        self.assertEqual(server['id'], volume['attachments'][0]['server_id'])
        self.assertTrue(self.demo_cloud.delete_server(server.id, wait=True))
        self.assertTrue(self.demo_cloud.delete_volume(volume.id, wait=True))
        self.assertIsNone(self.demo_cloud.get_server(server.id))
        self.assertIsNone(self.demo_cloud.get_volume(volume.id))

    def test_create_terminate_volume_image(self):
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            boot_from_volume=True,
            terminate_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server)
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        volume = self.demo_cloud.get_volume(volume_id)
        # We can either get None (if the volume delete was quick), or a volume
        # that is in the process of being deleted.
        if volume:
            self.assertEqual('deleting', volume.status)
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))

    def test_create_boot_from_volume_preexisting(self):
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.demo_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=None,
            flavor=self.flavor,
            boot_volume=volume,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server, volume_id=volume['id'])
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        self.addCleanup(self.demo_cloud.delete_volume, volume_id)
        volume = self.demo_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertEqual(True, volume['bootable'])
        self.assertEqual([], volume['attachments'])
        self.assertTrue(self.demo_cloud.delete_volume(volume_id))
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))
        self.assertIsNone(self.demo_cloud.get_volume(volume_id))

    def test_create_boot_attach_volume(self):
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.demo_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        self.addCleanup(self.demo_cloud.delete_volume, volume['id'])
        server = self.demo_cloud.create_server(
            name=self.server_name,
            flavor=self.flavor,
            image=self.image,
            boot_from_volume=False,
            volumes=[volume],
            wait=True)
        volume_id = self._assert_volume_attach(
            server, volume_id=volume['id'], image={'id': self.image['id']})
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        volume = self.demo_cloud.get_volume(volume_id)
        self.assertIsNotNone(volume)
        self.assertEqual(volume['name'], volume['display_name'])
        self.assertEqual([], volume['attachments'])
        self.assertTrue(self.demo_cloud.delete_volume(volume_id))
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))
        self.assertIsNone(self.demo_cloud.get_volume(volume_id))

    def test_create_boot_from_volume_preexisting_terminate(self):
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        volume = self.demo_cloud.create_volume(
            size=1, name=self.server_name, image=self.image, wait=True)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=None,
            flavor=self.flavor,
            boot_volume=volume,
            terminate_volume=True,
            volume_size=1,
            wait=True)
        volume_id = self._assert_volume_attach(server, volume_id=volume['id'])
        self.assertTrue(
            self.demo_cloud.delete_server(self.server_name, wait=True))
        volume = self.demo_cloud.get_volume(volume_id)
        # We can either get None (if the volume delete was quick), or a volume
        # that is in the process of being deleted.
        if volume:
            self.assertEqual('deleting', volume.status)
        self.assertIsNone(self.demo_cloud.get_server(self.server_name))

    def test_create_image_snapshot_wait_active(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        server = self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            admin_pass='sheiqu9loegahSh',
            wait=True)
        image = self.demo_cloud.create_image_snapshot('test-snapshot', server,
                                                      wait=True)
        self.addCleanup(self.demo_cloud.delete_image, image['id'])
        self.assertEqual('active', image['status'])

    def test_set_and_delete_metadata(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        self.demo_cloud.set_server_metadata(self.server_name,
                                            {'key1': 'value1',
                                             'key2': 'value2'})
        updated_server = self.demo_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1', 'key2': 'value2'}.items()))

        self.demo_cloud.set_server_metadata(self.server_name,
                                            {'key2': 'value3'})
        updated_server = self.demo_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1', 'key2': 'value3'}.items()))

        self.demo_cloud.delete_server_metadata(self.server_name, ['key2'])
        updated_server = self.demo_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()),
                         set({'key1': 'value1'}.items()))

        self.demo_cloud.delete_server_metadata(self.server_name, ['key1'])
        updated_server = self.demo_cloud.get_server(self.server_name)
        self.assertEqual(set(updated_server.metadata.items()), set([]))

        self.assertRaises(
            exc.OpenStackCloudException,
            self.demo_cloud.delete_server_metadata,
            self.server_name, ['key1'])

    def test_update_server(self):
        self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
        self.demo_cloud.create_server(
            name=self.server_name,
            image=self.image,
            flavor=self.flavor,
            wait=True)
        server_updated = self.demo_cloud.update_server(
            self.server_name,
            name='new_name'
        )
        self.assertEqual('new_name', server_updated['name'])



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_devstack
-------------

Throw errors if we do not actually detect the services we're supposed to.
"""
import os

from testscenarios import load_tests_apply_scenarios as load_tests  # noqa

from shade.tests.functional import base


class TestDevstack(base.BaseFunctionalTestCase):

    scenarios = [
        ('designate', dict(env='DESIGNATE', service='dns')),
        ('heat', dict(env='HEAT', service='orchestration')),
        ('magnum', dict(env='MAGNUM', service='container-infra')),
        ('neutron', dict(env='NEUTRON', service='network')),
        ('swift', dict(env='SWIFT', service='object-store')),
    ]

    def test_has_service(self):
        if os.environ.get('SHADE_HAS_{env}'.format(env=self.env), '0') == '1':
            self.assertTrue(self.demo_cloud.has_service(self.service))


class TestKeystoneVersion(base.BaseFunctionalTestCase):

    def test_keystone_version(self):
        use_keystone_v2 = os.environ.get('SHADE_USE_KEYSTONE_V2', False)
        if use_keystone_v2 and use_keystone_v2 != '0':
            self.assertEqual('2.0', self.identity_version)
        else:
            self.assertEqual('3', self.identity_version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_create_server
----------------------------------

Tests for the `create_server` command.
"""

from mock import patch, Mock
import mock
from shade import meta
from shade import OpenStackCloud
from shade.exc import (OpenStackCloudException, OpenStackCloudTimeout)
from shade.tests import fakes
from shade.tests.unit import base


class TestCreateServer(base.RequestsMockTestCase):

    def test_create_server_with_create_exception(self):
        """
        Test that an exception in the novaclient create raises an exception in
        create_server.
        """
        with patch("shade.OpenStackCloud.nova_client"):
            config = {
                "servers.create.side_effect": Exception("exception"),
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudException, self.cloud.create_server,
                'server-name', {'id': 'image-id'}, {'id': 'flavor-id'})

    def test_create_server_with_get_exception(self):
        """
        Test that an exception when attempting to get the server instance via
        the novaclient raises an exception in create_server.
        """

        with patch("shade.OpenStackCloud.nova_client"):
            config = {
                "servers.create.return_value": Mock(status="BUILD"),
                "servers.get.side_effect": Exception("exception")
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudException, self.cloud.create_server,
                'server-name', {'id': 'image-id'}, {'id': 'flavor-id'})

    def test_create_server_with_server_error(self):
        """
        Test that a server error before we return or begin waiting for the
        server instance spawn raises an exception in create_server.
        """
        build_server = fakes.FakeServer('1234', '', 'BUILD')
        error_server = fakes.FakeServer('1234', '', 'ERROR')
        with patch("shade.OpenStackCloud.nova_client"):
            config = {
                "servers.create.return_value": build_server,
                "servers.get.return_value": error_server,
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudException, self.cloud.create_server,
                'server-name', {'id': 'image-id'}, {'id': 'flavor-id'})

    def test_create_server_wait_server_error(self):
        """
        Test that a server error while waiting for the server to spawn
        raises an exception in create_server.
        """
        with patch("shade.OpenStackCloud.nova_client"):
            build_server = fakes.FakeServer('1234', '', 'BUILD')
            error_server = fakes.FakeServer('1234', '', 'ERROR')
            fake_floating_ip = fakes.FakeFloatingIP('1234', 'ippool',
                                                    '1.1.1.1', '2.2.2.2',
                                                    '5678')
            config = {
                "servers.create.return_value": build_server,
                "servers.get.return_value": build_server,
                "servers.list.side_effect": [
                    [build_server], [error_server]],
                "floating_ips.list.return_value": [fake_floating_ip]
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudException,
                self.cloud.create_server,
                'server-name', dict(id='image-id'),
                dict(id='flavor-id'), wait=True)

    def test_create_server_with_timeout(self):
        """
        Test that a timeout while waiting for the server to spawn raises an
        exception in create_server.
        """
        with patch("shade.OpenStackCloud.nova_client"):
            fake_server = fakes.FakeServer('1234', '', 'BUILD')
            config = {
                "servers.create.return_value": fake_server,
                "servers.get.return_value": fake_server,
                "servers.list.return_value": [fake_server],
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudTimeout,
                self.cloud.create_server,
                'server-name',
                dict(id='image-id'), dict(id='flavor-id'),
                wait=True, timeout=0.01)

    def test_create_server_no_wait(self):
        """
        Test that create_server with no wait and no exception in the
        novaclient create call returns the server instance.
        """
        with patch("shade.OpenStackCloud.nova_client"):
            fake_server = fakes.FakeServer('1234', '', 'BUILD')
            fake_floating_ip = fakes.FakeFloatingIP('1234', 'ippool',
                                                    '1.1.1.1', '2.2.2.2',
                                                    '5678')
            config = {
                "servers.create.return_value": fake_server,
                "servers.get.return_value": fake_server,
                "floating_ips.list.return_value": [fake_floating_ip]
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertEqual(
                self.cloud._normalize_server(
                    meta.obj_to_dict(fake_server)),
                self.cloud.create_server(
                    name='server-name',
                    image=dict(id='image=id'),
                    flavor=dict(id='flavor-id')))

    def test_create_server_with_admin_pass_no_wait(self):
        """
        Test that a server with an admin_pass passed returns the password
        """
        with patch("shade.OpenStackCloud.nova_client"):
            fake_server = fakes.FakeServer('1234', '', 'BUILD')
            fake_create_server = fakes.FakeServer('1234', '', 'BUILD',
                                                  adminPass='ooBootheiX0edoh')
            fake_floating_ip = fakes.FakeFloatingIP('1234', 'ippool',
                                                    '1.1.1.1', '2.2.2.2',
                                                    '5678')
            config = {
                "servers.create.return_value": fake_create_server,
                "servers.get.return_value": fake_server,
                "floating_ips.list.return_value": [fake_floating_ip]
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertEqual(
                self.cloud._normalize_server(
                    meta.obj_to_dict(fake_create_server)),
                self.cloud.create_server(
                    name='server-name', image=dict(id='image=id'),
                    flavor=dict(id='flavor-id'), admin_pass='ooBootheiX0edoh'))

    @patch.object(OpenStackCloud, "wait_for_server")
    @patch.object(OpenStackCloud, "nova_client")
    def test_create_server_with_admin_pass_wait(self, mock_nova, mock_wait):
        """
        Test that a server with an admin_pass passed returns the password
        """
        fake_server = fakes.FakeServer('1234', '', 'BUILD')
        fake_server_with_pass = fakes.FakeServer('1234', '', 'BUILD',
                                                 adminPass='ooBootheiX0edoh')

        mock_nova.servers.create.return_value = fake_server
        mock_nova.servers.get.return_value = fake_server
        # The wait returns non-password server
        mock_wait.return_value = self.cloud._normalize_server(
            meta.obj_to_dict(fake_server))

        server = self.cloud.create_server(
            name='server-name', image=dict(id='image-id'),
            flavor=dict(id='flavor-id'),
            admin_pass='ooBootheiX0edoh', wait=True)

        # Assert that we did wait
        self.assertTrue(mock_wait.called)

        # Even with the wait, we should still get back a passworded server
        self.assertEqual(
            server,
            self.cloud._normalize_server(
                meta.obj_to_dict(fake_server_with_pass))
        )

    @patch.object(OpenStackCloud, "get_active_server")
    @patch.object(OpenStackCloud, "get_server")
    def test_wait_for_server(self, mock_get_server, mock_get_active_server):
        """
        Test that waiting for a server returns the server instance when
        its status changes to "ACTIVE".
        """
        building_server = {'id': 'fake_server_id', 'status': 'BUILDING'}
        active_server = {'id': 'fake_server_id', 'status': 'ACTIVE'}

        mock_get_server.side_effect = iter([building_server, active_server])
        mock_get_active_server.side_effect = iter([
            building_server, active_server])

        server = self.cloud.wait_for_server(building_server)

        self.assertEqual(2, mock_get_server.call_count)
        mock_get_server.assert_has_calls([
            mock.call(building_server['id']),
            mock.call(active_server['id']),
        ])

        self.assertEqual(2, mock_get_active_server.call_count)
        mock_get_active_server.assert_has_calls([
            mock.call(server=building_server, reuse=True, auto_ip=True,
                      ips=None, ip_pool=None, wait=True, timeout=mock.ANY,
                      nat_destination=None),
            mock.call(server=active_server, reuse=True, auto_ip=True,
                      ips=None, ip_pool=None, wait=True, timeout=mock.ANY,
                      nat_destination=None),
        ])

        self.assertEqual('ACTIVE', server['status'])

    @patch.object(OpenStackCloud, 'wait_for_server')
    @patch.object(OpenStackCloud, 'nova_client')
    def test_create_server_wait(self, mock_nova, mock_wait):
        """
        Test that create_server with a wait actually does the wait.
        """
        fake_server = {'id': 'fake_server_id', 'status': 'BUILDING'}
        mock_nova.servers.create.return_value = fake_server

        self.cloud.create_server(
            'server-name',
            dict(id='image-id'), dict(id='flavor-id'), wait=True),

        mock_wait.assert_called_once_with(
            fake_server, auto_ip=True, ips=None,
            ip_pool=None, reuse=True, timeout=180,
            nat_destination=None,
        )

    @patch('time.sleep')
    def test_create_server_no_addresses(self, mock_sleep):
        """
        Test that create_server with a wait throws an exception if the
        server doesn't have addresses.
        """
        with patch("shade.OpenStackCloud.nova_client"):
            build_server = fakes.FakeServer('1234', '', 'BUILD')
            fake_server = fakes.FakeServer('1234', '', 'ACTIVE')
            fake_floating_ip = fakes.FakeFloatingIP('1234', 'ippool',
                                                    '1.1.1.1', '2.2.2.2',
                                                    '5678')
            config = {
                "servers.create.return_value": build_server,
                "servers.get.return_value": [build_server, None],
                "servers.list.side_effect": [
                    [build_server], [fake_server]],
                "servers.delete.return_value": None,
                "floating_ips.list.return_value": [fake_floating_ip]
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.cloud._SERVER_AGE = 0
            with patch.object(OpenStackCloud, "add_ips_to_server",
                              return_value=fake_server):
                self.assertRaises(
                    OpenStackCloudException, self.cloud.create_server,
                    'server-name', {'id': 'image-id'}, {'id': 'flavor-id'},
                    wait=True)

    @patch('shade.OpenStackCloud.nova_client')
    @patch('shade.OpenStackCloud.get_network')
    def test_create_server_network_with_no_nics(self, mock_get_network,
                                                mock_nova):
        """
        Verify that if 'network' is supplied, and 'nics' is not, that we
        attempt to get the network for the server.
        """
        self.cloud.create_server(
            'server-name',
            dict(id='image-id'), dict(id='flavor-id'), network='network-name')
        mock_get_network.assert_called_once_with(name_or_id='network-name')

    @patch('shade.OpenStackCloud.nova_client')
    @patch('shade.OpenStackCloud.get_network')
    def test_create_server_network_with_empty_nics(self,
                                                   mock_get_network,
                                                   mock_nova):
        """
        Verify that if 'network' is supplied, along with an empty 'nics' list,
        it's treated the same as if 'nics' were not included.
        """
        self.cloud.create_server(
            'server-name', dict(id='image-id'), dict(id='flavor-id'),
            network='network-name', nics=[])
        mock_get_network.assert_called_once_with(name_or_id='network-name')

    @mock.patch.object(OpenStackCloud, 'get_server_by_id')
    @mock.patch.object(OpenStackCloud, 'get_image')
    @mock.patch.object(OpenStackCloud, 'nova_client')
    def test_create_server_get_flavor_image(
            self, mock_nova, mock_image, mock_get_server_by_id):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        self.cloud.create_server(
            'server-name', 'image-id', 'vanilla',
            nics=[{'net-id': 'some-network'}])
        mock_image.assert_called_once()

        self.assert_calls()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import munch
import six

_IMAGE_FIELDS = (
    'checksum',
    'container_format',
    'direct_url',
    'disk_format',
    'file',
    'id',
    'name',
    'owner',
    'virtual_size',
)

_SERVER_FIELDS = (
    'accessIPv4',
    'accessIPv6',
    'addresses',
    'adminPass',
    'created',
    'key_name',
    'metadata',
    'networks',
    'private_v4',
    'public_v4',
    'public_v6',
    'security_groups',
    'status',
    'updated',
    'user_id',
)

_COMPUTE_LIMITS_FIELDS = (
    ('maxPersonality', 'max_personality'),
    ('maxPersonalitySize', 'max_personality_size'),
    ('maxServerGroupMembers', 'max_server_group_members'),
    ('maxServerGroups', 'max_server_groups'),
    ('maxServerMeta', 'max_server_meta'),
    ('maxTotalCores', 'max_total_cores'),
    ('maxTotalInstances', 'max_total_instances'),
    ('maxTotalKeypairs', 'max_total_keypairs'),
    ('maxTotalRAMSize', 'max_total_ram_size'),
    ('totalCoresUsed', 'total_cores_used'),
    ('totalInstancesUsed', 'total_instances_used'),
    ('totalRAMUsed', 'total_ram_used'),
    ('totalServerGroupsUsed', 'total_server_groups_used'),
)


_pushdown_fields = {
    'project': [
        'domain'
    ]
}


def _split_filters(obj_name='', filters=None, **kwargs):
    # Handle jmsepath filters
    if not filters:
        filters = {}
    if not isinstance(filters, dict):
        return {}, filters
    # Filter out None values from extra kwargs, because those are
    # defaults. If you want to search for things with None values,
    # they're going to need to go into the filters dict
    for (key, value) in kwargs.items():
        if value is not None:
            filters[key] = value
    pushdown = {}
    client = {}
    for (key, value) in filters.items():
        if key in _pushdown_fields.get(obj_name, {}):
            pushdown[key] = value
        else:
            client[key] = value
    return pushdown, client


def _to_bool(value):
    if isinstance(value, six.string_types):
        if not value:
            return False
        prospective = value.lower().capitalize()
        return prospective == 'True'
    return bool(value)


def _pop_int(resource, key):
    return int(resource.pop(key, 0) or 0)


def _pop_float(resource, key):
    return float(resource.pop(key, 0) or 0)


def _pop_or_get(resource, key, default, strict):
    if strict:
        return resource.pop(key, default)
    else:
        return resource.get(key, default)


class Normalizer(object):
    '''Mix-in class to provide the normalization functions.

    This is in a separate class just for on-disk source code organization
    reasons.
    '''

    def _normalize_compute_limits(self, limits, project_id=None):
        """ Normalize a limits object.

        Limits modified in this method and shouldn't be modified afterwards.
        """

        # Copy incoming limits because of shared dicts in unittests
        limits = limits['absolute'].copy()

        new_limits = munch.Munch()
        new_limits['location'] = self._get_current_location(
            project_id=project_id)

        for field in _COMPUTE_LIMITS_FIELDS:
            new_limits[field[1]] = limits.pop(field[0], None)

        new_limits['properties'] = limits.copy()

        return new_limits

    def _normalize_flavors(self, flavors):
        """ Normalize a list of flavor objects """
        ret = []
        for flavor in flavors:
            ret.append(self._normalize_flavor(flavor))
        return ret

    def _normalize_flavor(self, flavor):
        """ Normalize a flavor object """
        new_flavor = munch.Munch()

        # Copy incoming group because of shared dicts in unittests
        flavor = flavor.copy()

        # Discard noise
        flavor.pop('links', None)
        flavor.pop('NAME_ATTR', None)
        flavor.pop('HUMAN_ID', None)
        flavor.pop('human_id', None)
        flavor.pop('request_ids', None)

        ephemeral = int(_pop_or_get(
            flavor, 'OS-FLV-EXT-DATA:ephemeral', 0, self.strict_mode))
        ephemeral = flavor.pop('ephemeral', ephemeral)
        is_public = _to_bool(_pop_or_get(
            flavor, 'os-flavor-access:is_public', True, self.strict_mode))
        is_public = _to_bool(flavor.pop('is_public', is_public))
        is_disabled = _to_bool(_pop_or_get(
            flavor, 'OS-FLV-DISABLED:disabled', False, self.strict_mode))
        extra_specs = _pop_or_get(
            flavor, 'OS-FLV-WITH-EXT-SPECS:extra_specs', {}, self.strict_mode)
        extra_specs = flavor.pop('extra_specs', extra_specs)
        extra_specs = munch.Munch(extra_specs)

        new_flavor['location'] = self.current_location
        new_flavor['id'] = flavor.pop('id')
        new_flavor['name'] = flavor.pop('name')
        new_flavor['is_public'] = is_public
        new_flavor['is_disabled'] = is_disabled
        new_flavor['ram'] = _pop_int(flavor, 'ram')
        new_flavor['vcpus'] = _pop_int(flavor, 'vcpus')
        new_flavor['disk'] = _pop_int(flavor, 'disk')
        new_flavor['ephemeral'] = ephemeral
        new_flavor['swap'] = _pop_int(flavor, 'swap')
        new_flavor['rxtx_factor'] = _pop_float(flavor, 'rxtx_factor')

        new_flavor['properties'] = flavor.copy()
        new_flavor['extra_specs'] = extra_specs

        # Backwards compat with nova - passthrough values
        if not self.strict_mode:
            for (k, v) in new_flavor['properties'].items():
                new_flavor.setdefault(k, v)

        return new_flavor

    def _normalize_images(self, images):
        ret = []
        for image in images:
            ret.append(self._normalize_image(image))
        return ret

    def _normalize_image(self, image):
        new_image = munch.Munch(
            location=self._get_current_location(project_id=image.get('owner')))

        # This copy is to keep things from getting epically weird in tests
        image = image.copy()

        image.pop('links', None)
        image.pop('NAME_ATTR', None)
        image.pop('HUMAN_ID', None)
        image.pop('human_id', None)

        properties = image.pop('properties', {})
        visibility = image.pop('visibility', None)
        protected = _to_bool(image.pop('protected', False))

        if visibility:
            is_public = (visibility == 'public')
        else:
            is_public = image.pop('is_public', False)
            visibility = 'public' if is_public else 'private'

        new_image['size'] = image.pop('OS-EXT-IMG-SIZE:size', 0)
        new_image['size'] = image.pop('size', new_image['size'])

        new_image['min_ram'] = image.pop('minRam', 0)
        new_image['min_ram'] = image.pop('min_ram', new_image['min_ram'])

        new_image['min_disk'] = image.pop('minDisk', 0)
        new_image['min_disk'] = image.pop('min_disk', new_image['min_disk'])

        new_image['created_at'] = image.pop('created', '')
        new_image['created_at'] = image.pop(
            'created_at', new_image['created_at'])

        new_image['updated_at'] = image.pop('updated', '')
        new_image['updated_at'] = image.pop(
            'updated_at', new_image['updated_at'])

        for field in _IMAGE_FIELDS:
            new_image[field] = image.pop(field, None)

        new_image['tags'] = image.pop('tags', [])
        new_image['status'] = image.pop('status').lower()
        for field in ('min_ram', 'min_disk', 'size', 'virtual_size'):
            new_image[field] = _pop_int(new_image, field)
        new_image['is_protected'] = protected
        new_image['locations'] = image.pop('locations', [])

        metadata = image.pop('metadata', {})
        for key, val in metadata.items():
            properties.setdefault(key, val)

        for key, val in image.items():
            properties.setdefault(key, val)
        new_image['properties'] = properties
        new_image['is_public'] = is_public
        new_image['visibility'] = visibility

        # Backwards compat with glance
        if not self.strict_mode:
            for key, val in properties.items():
                new_image[key] = val
            new_image['protected'] = protected
            new_image['metadata'] = properties
            new_image['created'] = new_image['created_at']
            new_image['updated'] = new_image['updated_at']
            new_image['minDisk'] = new_image['min_disk']
            new_image['minRam'] = new_image['min_ram']
        return new_image

    def _normalize_secgroups(self, groups):
        """Normalize the structure of security groups

        This makes security group dicts, as returned from nova, look like the
        security group dicts as returned from neutron. This does not make them
        look exactly the same, but it's pretty close.

        :param list groups: A list of security group dicts.

        :returns: A list of normalized dicts.
        """
        ret = []
        for group in groups:
            ret.append(self._normalize_secgroup(group))
        return ret

    def _normalize_secgroup(self, group):

        ret = munch.Munch()
        # Copy incoming group because of shared dicts in unittests
        group = group.copy()

        rules = self._normalize_secgroup_rules(
            group.pop('security_group_rules', group.pop('rules', [])))
        project_id = group.pop('tenant_id', '')
        project_id = group.pop('project_id', project_id)

        ret['location'] = self._get_current_location(project_id=project_id)
        ret['id'] = group.pop('id')
        ret['name'] = group.pop('name')
        ret['security_group_rules'] = rules
        ret['description'] = group.pop('description')
        ret['properties'] = group

        # Backwards compat with Neutron
        if not self.strict_mode:
            ret['tenant_id'] = project_id
            ret['project_id'] = project_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)

        return ret

    def _normalize_secgroup_rules(self, rules):
        """Normalize the structure of nova security group rules

        Note that nova uses -1 for non-specific port values, but neutron
        represents these with None.

        :param list rules: A list of security group rule dicts.

        :returns: A list of normalized dicts.
        """
        ret = []
        for rule in rules:
            ret.append(self._normalize_secgroup_rule(rule))
        return ret

    def _normalize_secgroup_rule(self, rule):
        ret = munch.Munch()
        # Copy incoming rule because of shared dicts in unittests
        rule = rule.copy()

        ret['id'] = rule.pop('id')
        ret['direction'] = rule.pop('direction', 'ingress')
        ret['ethertype'] = rule.pop('ethertype', 'IPv4')
        port_range_min = rule.get(
            'port_range_min', rule.pop('from_port', None))
        if port_range_min == -1:
            port_range_min = None
        if port_range_min is not None:
            port_range_min = int(port_range_min)
        ret['port_range_min'] = port_range_min
        port_range_max = rule.pop(
            'port_range_max', rule.pop('to_port', None))
        if port_range_max == -1:
            port_range_max = None
        if port_range_min is not None:
            port_range_min = int(port_range_min)
        ret['port_range_max'] = port_range_max
        ret['protocol'] = rule.pop('protocol', rule.pop('ip_protocol', None))
        ret['remote_ip_prefix'] = rule.pop(
            'remote_ip_prefix', rule.pop('ip_range', {}).get('cidr', None))
        ret['security_group_id'] = rule.pop(
            'security_group_id', rule.pop('parent_group_id', None))
        ret['remote_group_id'] = rule.pop('remote_group_id', None)
        project_id = rule.pop('tenant_id', '')
        project_id = rule.pop('project_id', project_id)
        ret['location'] = self._get_current_location(project_id=project_id)
        ret['properties'] = rule

        # Backwards compat with Neutron
        if not self.strict_mode:
            ret['tenant_id'] = project_id
            ret['project_id'] = project_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)
        return ret

    def _normalize_servers(self, servers):
        # Here instead of _utils because we need access to region and cloud
        # name from the cloud object
        ret = []
        for server in servers:
            ret.append(self._normalize_server(server))
        return ret

    def _normalize_server(self, server):
        ret = munch.Munch()
        # Copy incoming server because of shared dicts in unittests
        server = server.copy()

        server.pop('links', None)
        server.pop('NAME_ATTR', None)
        server.pop('HUMAN_ID', None)
        server.pop('human_id', None)

        ret['id'] = server.pop('id')
        ret['name'] = server.pop('name')

        server['flavor'].pop('links', None)
        ret['flavor'] = server.pop('flavor')

        # OpenStack can return image as a string when you've booted
        # from volume
        if str(server['image']) != server['image']:
            server['image'].pop('links', None)
        ret['image'] = server.pop('image')

        project_id = server.pop('tenant_id', '')
        project_id = server.pop('project_id', project_id)

        az = _pop_or_get(
            server, 'OS-EXT-AZ:availability_zone', None, self.strict_mode)
        ret['location'] = self._get_current_location(
            project_id=project_id, zone=az)

        # Ensure volumes is always in the server dict, even if empty
        ret['volumes'] = _pop_or_get(
            server, 'os-extended-volumes:volumes_attached',
            [], self.strict_mode)

        config_drive = server.pop('config_drive', False)
        ret['has_config_drive'] = _to_bool(config_drive)

        host_id = server.pop('hostId', None)
        ret['host_id'] = host_id

        ret['progress'] = _pop_int(server, 'progress')

        # Leave these in so that the general properties handling works
        ret['disk_config'] = _pop_or_get(
            server, 'OS-DCF:diskConfig', None, self.strict_mode)
        for key in (
                'OS-EXT-STS:power_state',
                'OS-EXT-STS:task_state',
                'OS-EXT-STS:vm_state',
                'OS-SRV-USG:launched_at',
                'OS-SRV-USG:terminated_at'):
            short_key = key.split(':')[1]
            ret[short_key] = _pop_or_get(server, key, None, self.strict_mode)

        for field in _SERVER_FIELDS:
            ret[field] = server.pop(field, None)
        ret['interface_ip'] = ''

        ret['properties'] = server.copy()

        # Backwards compat
        if not self.strict_mode:
            ret['hostId'] = host_id
            ret['config_drive'] = config_drive
            ret['project_id'] = project_id
            ret['tenant_id'] = project_id
            ret['region'] = self.region_name
            ret['cloud'] = self.name
            ret['az'] = az
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)
        return ret

    def _normalize_floating_ips(self, ips):
        """Normalize the structure of floating IPs

        Unfortunately, not all the Neutron floating_ip attributes are available
        with Nova and not all Nova floating_ip attributes are available with
        Neutron.
        This function extract attributes that are common to Nova and Neutron
        floating IP resource.
        If the whole structure is needed inside shade, shade provides private
        methods that returns "original" objects (e.g.
        _neutron_allocate_floating_ip)

        :param list ips: A list of Neutron floating IPs.

        :returns:
            A list of normalized dicts with the following attributes::

            [
              {
                "id": "this-is-a-floating-ip-id",
                "fixed_ip_address": "192.0.2.10",
                "floating_ip_address": "198.51.100.10",
                "network": "this-is-a-net-or-pool-id",
                "attached": True,
                "status": "ACTIVE"
              }, ...
            ]

        """
        return [
            self._normalize_floating_ip(ip) for ip in ips
        ]

    def _normalize_floating_ip(self, ip):
        ret = munch.Munch()

        # Copy incoming floating ip because of shared dicts in unittests
        ip = ip.copy()

        fixed_ip_address = ip.pop('fixed_ip_address', ip.pop('fixed_ip', None))
        floating_ip_address = ip.pop('floating_ip_address', ip.pop('ip', None))
        network_id = ip.pop(
            'floating_network_id', ip.pop('network', ip.pop('pool', None)))
        project_id = ip.pop('tenant_id', '')
        project_id = ip.pop('project_id', project_id)

        instance_id = ip.pop('instance_id', None)
        router_id = ip.pop('router_id', None)
        id = ip.pop('id')
        port_id = ip.pop('port_id', None)
        created_at = ip.pop('created_at', None)
        updated_at = ip.pop('updated_at', None)
        # Note - description may not always be on the underlying cloud.
        # Normalizing it here is easy - what do we do when people want to
        # set a description?
        description = ip.pop('description', '')
        revision_number = ip.pop('revision_number', None)

        if self._use_neutron_floating():
            attached = bool(port_id)
            status = ip.pop('status', 'UNKNOWN')
        else:
            attached = bool(instance_id)
            # In neutron's terms, Nova floating IPs are always ACTIVE
            status = 'ACTIVE'

        ret = munch.Munch(
            attached=attached,
            fixed_ip_address=fixed_ip_address,
            floating_ip_address=floating_ip_address,
            id=id,
            location=self._get_current_location(project_id=project_id),
            network=network_id,
            port=port_id,
            router=router_id,
            status=status,
            created_at=created_at,
            updated_at=updated_at,
            description=description,
            revision_number=revision_number,
            properties=ip.copy(),
        )
        # Backwards compat
        if not self.strict_mode:
            ret['port_id'] = port_id
            ret['router_id'] = router_id
            ret['project_id'] = project_id
            ret['tenant_id'] = project_id
            ret['floating_network_id'] = network_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)

        return ret

    def _normalize_projects(self, projects):
        """Normalize the structure of projects

        This makes tenants from keystone v2 look like projects from v3.

        :param list projects: A list of projects to normalize

        :returns: A list of normalized dicts.
        """
        ret = []
        for project in projects:
            ret.append(self._normalize_project(project))
        return ret

    def _normalize_project(self, project):

        # Copy incoming project because of shared dicts in unittests
        project = project.copy()

        # Discard noise
        project.pop('links', None)
        project.pop('NAME_ATTR', None)
        project.pop('HUMAN_ID', None)
        project.pop('human_id', None)

        # In both v2 and v3
        project_id = project.pop('id')
        name = project.pop('name', '')
        description = project.pop('description', '')
        is_enabled = project.pop('enabled', True)

        # Projects are global - strip region
        location = self._get_current_location(project_id=project_id)
        location['region_name'] = None

        # v3 additions
        domain_id = project.pop('domain_id', 'default')
        parent_id = project.pop('parent_id', None)
        is_domain = project.pop('is_domain', False)

        # Projects have a special relationship with location
        location['project']['domain_id'] = domain_id
        location['project']['domain_name'] = None
        location['project']['name'] = None
        location['project']['id'] = parent_id

        ret = munch.Munch(
            location=location,
            id=project_id,
            name=name,
            description=description,
            is_enabled=is_enabled,
            is_domain=is_domain,
            properties=project.copy()
        )

        # Backwards compat
        if not self.strict_mode:
            ret['enabled'] = is_enabled
            ret['domain_id'] = domain_id
            ret['parent_id'] = parent_id
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)

        return ret

    def _normalize_volume_type_access(self, volume_type_access):

        volume_type_access = volume_type_access.copy()

        volume_type_id = volume_type_access.pop('volume_type_id')
        project_id = volume_type_access.pop('project_id')
        ret = munch.Munch(
            location=self.current_location,
            project_id=project_id,
            volume_type_id=volume_type_id,
            properties=volume_type_access.copy(),
        )
        return ret

    def _normalize_volume_type_accesses(self, volume_type_accesses):
        ret = []
        for volume_type_access in volume_type_accesses:
            ret.append(self._normalize_volume_type_access(volume_type_access))
        return ret

    def _normalize_volume_type(self, volume_type):

        volume_type = volume_type.copy()

        volume_id = volume_type.pop('id')
        description = volume_type.pop('description', None)
        name = volume_type.pop('name', None)
        old_is_public = volume_type.pop('os-volume-type-access:is_public',
                                        False)
        is_public = volume_type.pop('is_public', old_is_public)
        qos_specs_id = volume_type.pop('qos_specs_id', None)
        extra_specs = volume_type.pop('extra_specs', {})
        ret = munch.Munch(
            location=self.current_location,
            is_public=is_public,
            id=volume_id,
            name=name,
            description=description,
            qos_specs_id=qos_specs_id,
            extra_specs=extra_specs,
            properties=volume_type.copy(),
        )
        return ret

    def _normalize_volume_types(self, volume_types):
        ret = []
        for volume in volume_types:
            ret.append(self._normalize_volume_type(volume))
        return ret

    def _normalize_volumes(self, volumes):
        """Normalize the structure of volumes

        This makes tenants from cinder v1 look like volumes from v2.

        :param list projects: A list of volumes to normalize

        :returns: A list of normalized dicts.
        """
        ret = []
        for volume in volumes:
            ret.append(self._normalize_volume(volume))
        return ret

    def _normalize_volume(self, volume):

        volume = volume.copy()

        # Discard noise
        volume.pop('links', None)
        volume.pop('NAME_ATTR', None)
        volume.pop('HUMAN_ID', None)
        volume.pop('human_id', None)

        volume_id = volume.pop('id')

        name = volume.pop('display_name', None)
        name = volume.pop('name', name)

        description = volume.pop('display_description', None)
        description = volume.pop('description', description)

        is_bootable = _to_bool(volume.pop('bootable', True))
        is_encrypted = _to_bool(volume.pop('encrypted', False))
        can_multiattach = _to_bool(volume.pop('multiattach', False))

        project_id = _pop_or_get(
            volume, 'os-vol-tenant-attr:tenant_id', None, self.strict_mode)
        az = volume.pop('availability_zone', None)

        location = self._get_current_location(project_id=project_id, zone=az)

        host = _pop_or_get(
            volume, 'os-vol-host-attr:host', None, self.strict_mode)
        replication_extended_status = _pop_or_get(
            volume, 'os-volume-replication:extended_status',
            None, self.strict_mode)

        migration_status = _pop_or_get(
            volume, 'os-vol-mig-status-attr:migstat', None, self.strict_mode)
        migration_status = volume.pop('migration_status', migration_status)
        _pop_or_get(volume, 'user_id', None, self.strict_mode)
        source_volume_id = _pop_or_get(
            volume, 'source_volid', None, self.strict_mode)
        replication_driver = _pop_or_get(
            volume, 'os-volume-replication:driver_data',
            None, self.strict_mode)

        ret = munch.Munch(
            location=location,
            id=volume_id,
            name=name,
            description=description,
            size=_pop_int(volume, 'size'),
            attachments=volume.pop('attachments', []),
            status=volume.pop('status'),
            migration_status=migration_status,
            host=host,
            replication_driver=replication_driver,
            replication_status=volume.pop('replication_status', None),
            replication_extended_status=replication_extended_status,
            snapshot_id=volume.pop('snapshot_id', None),
            created_at=volume.pop('created_at'),
            updated_at=volume.pop('updated_at', None),
            source_volume_id=source_volume_id,
            consistencygroup_id=volume.pop('consistencygroup_id', None),
            volume_type=volume.pop('volume_type', None),
            metadata=volume.pop('metadata', {}),
            is_bootable=is_bootable,
            is_encrypted=is_encrypted,
            can_multiattach=can_multiattach,
            properties=volume.copy(),
        )

        # Backwards compat
        if not self.strict_mode:
            ret['display_name'] = name
            ret['display_description'] = description
            ret['bootable'] = is_bootable
            ret['encrypted'] = is_encrypted
            ret['multiattach'] = can_multiattach
            ret['availability_zone'] = az
            for key, val in ret['properties'].items():
                ret.setdefault(key, val)
        return ret

    def _normalize_usage(self, usage):
        """ Normalize a usage object """

        # Discard noise
        usage.pop('links', None)
        usage.pop('NAME_ATTR', None)
        usage.pop('HUMAN_ID', None)
        usage.pop('human_id', None)
        usage.pop('request_ids', None)

        return munch.Munch(usage)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

import shade
from shade.tests.unit import base
from shade.tests import fakes


class TestLimits(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    def test_get_compute_limits(self, mock_nova):
        self.cloud.get_compute_limits()

        mock_nova.limits.get.assert_called_once_with()

    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_other_get_compute_limits(self, mock_keystone, mock_nova):
        project = fakes.FakeProject('project_a')
        mock_keystone.tenants.list.return_value = [project]
        self.op_cloud.get_compute_limits(project)

        mock_nova.limits.get.assert_called_once_with(tenant_id='project_a')



# Copyright (c) 2016 IBM
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_project
----------------------------------

Functional tests for `shade` project resource.
"""

from shade.exc import OpenStackCloudException
from shade.tests.functional import base


class TestProject(base.BaseFunctionalTestCase):

    def setUp(self):
        super(TestProject, self).setUp()
        self.new_project_name = self.getUniqueString('project')
        self.addCleanup(self._cleanup_projects)

    def _cleanup_projects(self):
        exception_list = list()
        for p in self.operator_cloud.list_projects():
            if p['name'].startswith(self.new_project_name):
                try:
                    self.operator_cloud.delete_project(p['id'])
                except Exception as e:
                    exception_list.append(str(e))
                    continue
        if exception_list:
            raise OpenStackCloudException('\n'.join(exception_list))

    def test_create_project(self):
        project_name = self.new_project_name + '_create'

        params = {
            'name': project_name,
            'description': 'test_create_project',
        }
        if self.identity_version == '3':
            params['domain_id'] = \
                self.operator_cloud.get_domain('default')['id']

        project = self.operator_cloud.create_project(**params)

        self.assertIsNotNone(project)
        self.assertEqual(project_name, project['name'])
        self.assertEqual('test_create_project', project['description'])

    def test_update_project(self):
        project_name = self.new_project_name + '_update'

        params = {
            'name': project_name,
            'description': 'test_update_project',
        }
        if self.identity_version == '3':
            params['domain_id'] = \
                self.operator_cloud.get_domain('default')['id']

        project = self.operator_cloud.create_project(**params)
        updated_project = self.operator_cloud.update_project(project_name,
                                                             description='new')
        self.assertIsNotNone(updated_project)
        self.assertEqual(project['id'], updated_project['id'])
        self.assertEqual(project['name'], updated_project['name'])
        self.assertEqual(updated_project['description'], 'new')

    def test_delete_project(self):
        project_name = self.new_project_name + '_delete'
        params = {'name': project_name}
        if self.identity_version == '3':
            params['domain_id'] = \
                self.operator_cloud.get_domain('default')['id']
        project = self.operator_cloud.create_project(**params)
        self.assertIsNotNone(project)
        self.assertTrue(self.operator_cloud.delete_project(project['id']))

    def test_delete_project_not_found(self):
        self.assertFalse(self.operator_cloud.delete_project('doesNotExist'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_update_server
----------------------------------

Tests for the `update_server` command.
"""

from mock import patch, Mock
from shade import OpenStackCloud
from shade.exc import OpenStackCloudException
from shade.tests import fakes
from shade.tests.unit import base


class TestUpdateServer(base.TestCase):

    def test_update_server_with_update_exception(self):
        """
        Test that an exception in the novaclient update raises an exception in
        update_server.
        """
        with patch("shade.OpenStackCloud"):
            config = {
                "servers.update.side_effect": Exception("exception"),
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertRaises(
                OpenStackCloudException, self.cloud.update_server,
                'server-name')

    def test_update_server_name(self):
        """
        Test that update_server updates the name without raising any exception
        """
        with patch("shade.OpenStackCloud"):
            fake_server = fakes.FakeServer('1234', 'server-name', 'ACTIVE')
            fake_update_server = fakes.FakeServer('1234', 'server-name2',
                                                  'ACTIVE')
            fake_floating_ip = fakes.FakeFloatingIP('1234', 'ippool',
                                                    '1.1.1.1', '2.2.2.2',
                                                    '5678')
            config = {
                "servers.list.return_value": [fake_server],
                "servers.update.return_value": fake_update_server,
                "floating_ips.list.return_value": [fake_floating_ip]
            }
            OpenStackCloud.nova_client = Mock(**config)
            self.assertEqual(
                'server-name2',
                self.cloud.update_server(
                    'server-name', name='server-name2')['name'])



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
test_port
----------------------------------

Test port resource (managed by neutron)
"""

from mock import patch
from shade import OpenStackCloud
from shade.exc import OpenStackCloudException
from shade.tests.unit import base


class TestPort(base.TestCase):
    mock_neutron_port_create_rep = {
        'port': {
            'status': 'DOWN',
            'binding:host_id': '',
            'name': 'test-port-name',
            'allowed_address_pairs': [],
            'admin_state_up': True,
            'network_id': 'test-net-id',
            'tenant_id': 'test-tenant-id',
            'binding:vif_details': {},
            'binding:vnic_type': 'normal',
            'binding:vif_type': 'unbound',
            'device_owner': '',
            'mac_address': '50:1c:0d:e4:f0:0d',
            'binding:profile': {},
            'fixed_ips': [
                {
                    'subnet_id': 'test-subnet-id',
                    'ip_address': '29.29.29.29'
                }
            ],
            'id': 'test-port-id',
            'security_groups': [],
            'device_id': ''
        }
    }

    mock_neutron_port_update_rep = {
        'port': {
            'status': 'DOWN',
            'binding:host_id': '',
            'name': 'test-port-name-updated',
            'allowed_address_pairs': [],
            'admin_state_up': True,
            'network_id': 'test-net-id',
            'tenant_id': 'test-tenant-id',
            'binding:vif_details': {},
            'binding:vnic_type': 'normal',
            'binding:vif_type': 'unbound',
            'device_owner': '',
            'mac_address': '50:1c:0d:e4:f0:0d',
            'binding:profile': {},
            'fixed_ips': [
                {
                    'subnet_id': 'test-subnet-id',
                    'ip_address': '29.29.29.29'
                }
            ],
            'id': 'test-port-id',
            'security_groups': [],
            'device_id': ''
        }
    }

    mock_neutron_port_list_rep = {
        'ports': [
            {
                'status': 'ACTIVE',
                'binding:host_id': 'devstack',
                'name': 'first-port',
                'allowed_address_pairs': [],
                'admin_state_up': True,
                'network_id': '70c1db1f-b701-45bd-96e0-a313ee3430b3',
                'tenant_id': '',
                'extra_dhcp_opts': [],
                'binding:vif_details': {
                    'port_filter': True,
                    'ovs_hybrid_plug': True
                },
                'binding:vif_type': 'ovs',
                'device_owner': 'network:router_gateway',
                'mac_address': 'fa:16:3e:58:42:ed',
                'binding:profile': {},
                'binding:vnic_type': 'normal',
                'fixed_ips': [
                    {
                        'subnet_id': '008ba151-0b8c-4a67-98b5-0d2b87666062',
                        'ip_address': '172.24.4.2'
                    }
                ],
                'id': 'd80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
                'security_groups': [],
                'device_id': '9ae135f4-b6e0-4dad-9e91-3c223e385824'
            },
            {
                'status': 'ACTIVE',
                'binding:host_id': 'devstack',
                'name': '',
                'allowed_address_pairs': [],
                'admin_state_up': True,
                'network_id': 'f27aa545-cbdd-4907-b0c6-c9e8b039dcc2',
                'tenant_id': 'd397de8a63f341818f198abb0966f6f3',
                'extra_dhcp_opts': [],
                'binding:vif_details': {
                    'port_filter': True,
                    'ovs_hybrid_plug': True
                },
                'binding:vif_type': 'ovs',
                'device_owner': 'network:router_interface',
                'mac_address': 'fa:16:3e:bb:3c:e4',
                'binding:profile': {},
                'binding:vnic_type': 'normal',
                'fixed_ips': [
                    {
                        'subnet_id': '288bf4a1-51ba-43b6-9d0a-520e9005db17',
                        'ip_address': '10.0.0.1'
                    }
                ],
                'id': 'f71a6703-d6de-4be1-a91a-a570ede1d159',
                'security_groups': [],
                'device_id': '9ae135f4-b6e0-4dad-9e91-3c223e385824'
            }
        ]
    }

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_create_port(self, mock_neutron_client):
        mock_neutron_client.create_port.return_value = \
            self.mock_neutron_port_create_rep

        port = self.cloud.create_port(
            network_id='test-net-id', name='test-port-name',
            admin_state_up=True)

        mock_neutron_client.create_port.assert_called_with(
            body={'port': dict(network_id='test-net-id', name='test-port-name',
                               admin_state_up=True)})
        self.assertEqual(self.mock_neutron_port_create_rep['port'], port)

    def test_create_port_parameters(self):
        """Test that we detect invalid arguments passed to create_port"""
        self.assertRaises(
            TypeError, self.cloud.create_port,
            network_id='test-net-id', nome='test-port-name',
            stato_amministrativo_porta=True)

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_create_port_exception(self, mock_neutron_client):
        mock_neutron_client.create_port.side_effect = Exception('blah')

        self.assertRaises(
            OpenStackCloudException, self.cloud.create_port,
            network_id='test-net-id', name='test-port-name',
            admin_state_up=True)

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_update_port(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep
        mock_neutron_client.update_port.return_value = \
            self.mock_neutron_port_update_rep

        port = self.cloud.update_port(
            name_or_id='d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
            name='test-port-name-updated')

        mock_neutron_client.update_port.assert_called_with(
            port='d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
            body={'port': dict(name='test-port-name-updated')})
        self.assertEqual(self.mock_neutron_port_update_rep['port'], port)

    def test_update_port_parameters(self):
        """Test that we detect invalid arguments passed to update_port"""
        self.assertRaises(
            TypeError, self.cloud.update_port,
            name_or_id='test-port-id', nome='test-port-name-updated')

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_update_port_exception(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep
        mock_neutron_client.update_port.side_effect = Exception('blah')

        self.assertRaises(
            OpenStackCloudException, self.cloud.update_port,
            name_or_id='d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b',
            name='test-port-name-updated')

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_list_ports(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep

        ports = self.cloud.list_ports()

        mock_neutron_client.list_ports.assert_called_with()
        self.assertItemsEqual(self.mock_neutron_port_list_rep['ports'], ports)

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_list_ports_exception(self, mock_neutron_client):
        mock_neutron_client.list_ports.side_effect = Exception('blah')

        self.assertRaises(OpenStackCloudException, self.cloud.list_ports)

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_search_ports_by_id(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep

        ports = self.cloud.search_ports(
            name_or_id='f71a6703-d6de-4be1-a91a-a570ede1d159')

        mock_neutron_client.list_ports.assert_called_with()
        self.assertEqual(1, len(ports))
        self.assertEqual('fa:16:3e:bb:3c:e4', ports[0]['mac_address'])

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_search_ports_by_name(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep

        ports = self.cloud.search_ports(name_or_id='first-port')

        mock_neutron_client.list_ports.assert_called_with()
        self.assertEqual(1, len(ports))
        self.assertEqual('fa:16:3e:58:42:ed', ports[0]['mac_address'])

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_search_ports_not_found(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep

        ports = self.cloud.search_ports(name_or_id='non-existent')

        mock_neutron_client.list_ports.assert_called_with()
        self.assertEqual(0, len(ports))

    @patch.object(OpenStackCloud, 'neutron_client')
    def test_delete_port(self, mock_neutron_client):
        mock_neutron_client.list_ports.return_value = \
            self.mock_neutron_port_list_rep

        self.cloud.delete_port(name_or_id='first-port')

        mock_neutron_client.delete_port.assert_called_with(
            port='d80b1a3b-4fc1-49f3-952e-1e2ab7081d8b')



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

import ironicclient
from os_client_config import cloud_config
import shade
from shade.tests import base


class TestShadeOperatorNoAuth(base.TestCase):
    def setUp(self):
        """Setup Noauth OperatorCloud tests

        Setup the test to utilize no authentication and an endpoint
        URL in the auth data.  This is permits testing of the basic
        mechanism that enables Ironic noauth mode to be utilized with
        Shade.
        """
        super(TestShadeOperatorNoAuth, self).setUp()
        self.cloud_noauth = shade.operator_cloud(
            auth_type='admin_token',
            auth=dict(endpoint="http://localhost:6385"),
            validate=False,
        )

    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
    @mock.patch.object(ironicclient.client, 'Client')
    def test_ironic_noauth_selection_using_a_task(
            self, mock_client, get_session_mock):
        """Test noauth selection for Ironic in OperatorCloud

        Utilize a task to trigger the client connection attempt
        and evaluate if get_session_endpoint was called while the client
        was still called.

        We want session_endpoint to be called because we're storing the
        endpoint in a noauth token Session object now.
        """
        session_mock = mock.Mock()
        session_mock.get_endpoint.return_value = None
        session_mock.get_token.return_value = 'yankee'
        get_session_mock.return_value = session_mock

        self.cloud_noauth.patch_machine('name', {})
        self.assertTrue(get_session_mock.called)
        self.assertTrue(mock_client.called)



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures
import logging
import munch
from six import StringIO
import testtools
import testtools.content

_TRUE_VALUES = ('true', '1', 'yes')


class TestCase(testtools.TestCase):

    """Test case base class for all tests."""

    # A way to adjust slow test classes
    TIMEOUT_SCALING_FACTOR = 1.0

    def setUp(self):
        """Run before each test method to initialize test environment."""

        super(TestCase, self).setUp()
        test_timeout = int(os.environ.get('OS_TEST_TIMEOUT', 0))
        try:
            test_timeout = int(test_timeout * self.TIMEOUT_SCALING_FACTOR)
        except ValueError:
            # If timeout value is invalid do not set a timeout.
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        self.useFixture(fixtures.NestedTempfile())
        self.useFixture(fixtures.TempHomeDir())

        if os.environ.get('OS_STDOUT_CAPTURE') in _TRUE_VALUES:
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if os.environ.get('OS_STDERR_CAPTURE') in _TRUE_VALUES:
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))

        self._log_stream = StringIO()
        if os.environ.get('OS_ALWAYS_LOG') in _TRUE_VALUES:
            self.addCleanup(self.printLogs)
        else:
            self.addOnException(self.attachLogs)

        handler = logging.StreamHandler(self._log_stream)
        formatter = logging.Formatter('%(asctime)s %(name)-32s %(message)s')
        handler.setFormatter(formatter)

        logger = logging.getLogger('shade')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)

        # Enable HTTP level tracing
        logger = logging.getLogger('keystoneauth')
        logger.setLevel(logging.DEBUG)
        logger.addHandler(handler)
        logger.propagate = False

    def assertEqual(self, first, second, *args, **kwargs):
        '''Munch aware wrapper'''
        if isinstance(first, munch.Munch):
            first = first.toDict()
        if isinstance(second, munch.Munch):
            second = second.toDict()
        return super(TestCase, self).assertEqual(
            first, second, *args, **kwargs)

    def printLogs(self, *args):
        self._log_stream.seek(0)
        print(self._log_stream.read())

    def attachLogs(self, *args):
        def reader():
            self._log_stream.seek(0)
            while True:
                x = self._log_stream.read(4096)
                if not x:
                    break
                yield x.encode('utf8')
        content = testtools.content.content_from_reader(
            reader,
            testtools.content_type.UTF8_TEXT,
            False)
        self.addDetail('logging', content)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_compute
----------------------------------

Functional tests for `shade` image methods.
"""

import filecmp
import os
import tempfile

from shade.tests.functional import base
from shade.tests.functional.util import pick_image


class TestImage(base.BaseFunctionalTestCase):
    def setUp(self):
        super(TestImage, self).setUp()
        self.image = pick_image(self.demo_cloud.nova_client.images.list())

    def test_create_image(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write('\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True)
        finally:
            self.demo_cloud.delete_image(image_name, wait=True)

    def test_download_image(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        self.addCleanup(os.remove, test_image.name)
        test_image.write('\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        self.demo_cloud.create_image(
            name=image_name,
            filename=test_image.name,
            disk_format='raw',
            container_format='bare',
            min_disk=10,
            min_ram=1024,
            wait=True)
        self.addCleanup(self.demo_cloud.delete_image, image_name, wait=True)
        output = os.path.join(tempfile.gettempdir(), self.getUniqueString())
        self.demo_cloud.download_image(image_name, output)
        self.addCleanup(os.remove, output)
        self.assertTrue(filecmp.cmp(test_image.name, output),
                        "Downloaded contents don't match created image")

    def test_create_image_skip_duplicate(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write('\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            first_image = self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True)
            second_image = self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True)
            self.assertEqual(first_image.id, second_image.id)
        finally:
            self.demo_cloud.delete_image(image_name, wait=True)

    def test_create_image_force_duplicate(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write('\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        first_image = None
        second_image = None
        try:
            first_image = self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True)
            second_image = self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                allow_duplicates=True,
                wait=True)
            self.assertNotEqual(first_image.id, second_image.id)
        finally:
            if first_image:
                self.demo_cloud.delete_image(first_image.id, wait=True)
            if second_image:
                self.demo_cloud.delete_image(second_image.id, wait=True)

    def test_create_image_update_properties(self):
        test_image = tempfile.NamedTemporaryFile(delete=False)
        test_image.write('\0' * 1024 * 1024)
        test_image.close()
        image_name = self.getUniqueString('image')
        try:
            image = self.demo_cloud.create_image(
                name=image_name,
                filename=test_image.name,
                disk_format='raw',
                container_format='bare',
                min_disk=10,
                min_ram=1024,
                wait=True)
            self.demo_cloud.update_image_properties(
                image=image,
                name=image_name,
                foo='bar')
            image = self.demo_cloud.get_image(image_name)
            self.assertIn('foo', image.properties)
            self.assertEqual(image.properties['foo'], 'bar')
        finally:
            self.demo_cloud.delete_image(image_name, wait=True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import mock
import munch

import shade
import testtools
from shade.tests.unit import base


cluster_template_obj = munch.Munch(
    apiserver_port=None,
    uuid='fake-uuid',
    human_id=None,
    name='fake-cluster-template',
    server_type='vm',
    public=False,
    image_id='fake-image',
    tls_disabled=False,
    registry_enabled=False,
    coe='fake-coe',
    keypair_id='fake-key',
)

cluster_template_detail_obj = munch.Munch(
    links={},
    labels={},
    apiserver_port=None,
    uuid='fake-uuid',
    human_id=None,
    name='fake-cluster-template',
    server_type='vm',
    public=False,
    image_id='fake-image',
    tls_disabled=False,
    registry_enabled=False,
    coe='fake-coe',
    created_at='fake-date',
    updated_at=None,
    master_flavor_id=None,
    no_proxy=None,
    https_proxy=None,
    keypair_id='fake-key',
    docker_volume_size=1,
    external_network_id='public',
    cluster_distro='fake-distro',
    volume_driver=None,
    network_driver='fake-driver',
    fixed_network=None,
    flavor_id='fake-flavor',
    dns_nameserver='8.8.8.8',
)


class TestClusterTemplates(base.TestCase):

    def setUp(self):
        super(TestClusterTemplates, self).setUp()
        self.cloud = shade.openstack_cloud(validate=False)

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_list_cluster_templates_without_detail(self, mock_magnum):
        mock_magnum.baymodels.list.return_value = [
            cluster_template_obj]
        cluster_templates_list = self.cloud.list_cluster_templates()
        mock_magnum.baymodels.list.assert_called_with(detail=False)
        self.assertEqual(cluster_templates_list[0], cluster_template_obj)

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_list_cluster_templates_with_detail(self, mock_magnum):
        mock_magnum.baymodels.list.return_value = [
            cluster_template_detail_obj]
        cluster_templates_list = self.cloud.list_cluster_templates(detail=True)
        mock_magnum.baymodels.list.assert_called_with(detail=True)
        self.assertEqual(
            cluster_templates_list[0], cluster_template_detail_obj)

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_search_cluster_templates_by_name(self, mock_magnum):
        mock_magnum.baymodels.list.return_value = [
            cluster_template_obj]

        cluster_templates = self.cloud.search_cluster_templates(
            name_or_id='fake-cluster-template')
        mock_magnum.baymodels.list.assert_called_with(detail=False)

        self.assertEqual(1, len(cluster_templates))
        self.assertEqual('fake-uuid', cluster_templates[0]['uuid'])

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_search_cluster_templates_not_found(self, mock_magnum):
        mock_magnum.baymodels.list.return_value = [
            cluster_template_obj]

        cluster_templates = self.cloud.search_cluster_templates(
            name_or_id='non-existent')

        mock_magnum.baymodels.list.assert_called_with(detail=False)
        self.assertEqual(0, len(cluster_templates))

    @mock.patch.object(shade.OpenStackCloud, 'search_cluster_templates')
    def test_get_cluster_template(self, mock_search):
        mock_search.return_value = [cluster_template_obj, ]
        r = self.cloud.get_cluster_template('fake-cluster-template')
        self.assertIsNotNone(r)
        self.assertDictEqual(cluster_template_obj, r)

    @mock.patch.object(shade.OpenStackCloud, 'search_cluster_templates')
    def test_get_cluster_template_not_found(self, mock_search):
        mock_search.return_value = []
        r = self.cloud.get_cluster_template('doesNotExist')
        self.assertIsNone(r)

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_create_cluster_template(self, mock_magnum):
        self.cloud.create_cluster_template(
            name=cluster_template_obj.name,
            image_id=cluster_template_obj.image_id,
            keypair_id=cluster_template_obj.keypair_id,
            coe=cluster_template_obj.coe)
        mock_magnum.baymodels.create.assert_called_once_with(
            name=cluster_template_obj.name,
            image_id=cluster_template_obj.image_id,
            keypair_id=cluster_template_obj.keypair_id,
            coe=cluster_template_obj.coe
        )

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_create_cluster_template_exception(self, mock_magnum):
        mock_magnum.baymodels.create.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Error creating ClusterTemplate of name fake-cluster-template"
        ):
            self.cloud.create_cluster_template('fake-cluster-template')

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_delete_cluster_template(self, mock_magnum):
        mock_magnum.baymodels.list.return_value = [
            cluster_template_obj]
        self.cloud.delete_cluster_template('fake-uuid')
        mock_magnum.baymodels.delete.assert_called_once_with(
            'fake-uuid'
        )

    @mock.patch.object(shade.OpenStackCloud, 'magnum_client')
    def test_update_cluster_template(self, mock_magnum):
        new_name = 'new-cluster-template'
        mock_magnum.baymodels.list.return_value = [
            cluster_template_obj]
        self.cloud.update_cluster_template(
            'fake-uuid', 'replace', name=new_name)
        mock_magnum.baymodels.update.assert_called_once_with(
            'fake-uuid', [{'path': '/name', 'op': 'replace',
                           'value': 'new-cluster-template'}]
        )



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import testtools

import shade
from shade import meta
from shade.tests.unit import base
from shade.tests import fakes


domain_obj = fakes.FakeDomain(
    id='1',
    name='a-domain',
    description='A wonderful keystone domain',
    enabled=True,
)


class TestDomains(base.TestCase):

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_list_domains(self, mock_keystone):
        self.op_cloud.list_domains()
        self.assertTrue(mock_keystone.domains.list.called)

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_get_domain(self, mock_keystone):
        mock_keystone.domains.get.return_value = domain_obj
        domain = self.op_cloud.get_domain(domain_id='1234')
        self.assertFalse(mock_keystone.domains.list.called)
        self.assertTrue(mock_keystone.domains.get.called)
        self.assertEqual(domain['name'], 'a-domain')

    @mock.patch.object(shade._utils, '_get_entity')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_get_domain_with_name_or_id(self, mock_keystone, mock_get):
        self.op_cloud.get_domain(name_or_id='1234')
        mock_get.assert_called_once_with(mock.ANY,
                                         None, '1234')

    @mock.patch.object(shade._utils, 'normalize_domains')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_create_domain(self, mock_keystone, mock_normalize):
        mock_keystone.domains.create.return_value = domain_obj
        self.op_cloud.create_domain(
            domain_obj.name, domain_obj.description)
        mock_keystone.domains.create.assert_called_once_with(
            name=domain_obj.name, description=domain_obj.description,
            enabled=True)
        mock_normalize.assert_called_once_with([meta.obj_to_dict(domain_obj)])

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_create_domain_exception(self, mock_keystone):
        mock_keystone.domains.create.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to create domain domain_name"
        ):
            self.op_cloud.create_domain('domain_name')

    @mock.patch.object(shade.OperatorCloud, 'update_domain')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_delete_domain(self, mock_keystone, mock_update):
        mock_update.return_value = dict(id='update_domain_id')
        self.op_cloud.delete_domain('domain_id')
        mock_update.assert_called_once_with('domain_id', enabled=False)
        mock_keystone.domains.delete.assert_called_once_with(
            domain='update_domain_id')

    @mock.patch.object(shade.OperatorCloud, 'get_domain')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_delete_domain_name_or_id(self, mock_keystone, mock_get):
        self.op_cloud.update_domain(
            name_or_id='a-domain',
            name='new name',
            description='new description',
            enabled=False)
        mock_get.assert_called_once_with(None, 'a-domain')

    @mock.patch.object(shade.OperatorCloud, 'update_domain')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_delete_domain_exception(self, mock_keystone, mock_update):
        mock_keystone.domains.delete.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Failed to delete domain domain_id"
        ):
            self.op_cloud.delete_domain('domain_id')

    @mock.patch.object(shade._utils, 'normalize_domains')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_update_domain(self, mock_keystone, mock_normalize):
        mock_keystone.domains.update.return_value = domain_obj
        self.op_cloud.update_domain(
            'domain_id',
            name='new name',
            description='new description',
            enabled=False)
        mock_keystone.domains.update.assert_called_once_with(
            domain='domain_id', name='new name',
            description='new description', enabled=False)
        mock_normalize.assert_called_once_with(
            [meta.obj_to_dict(domain_obj)])

    @mock.patch.object(shade.OperatorCloud, 'get_domain')
    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_update_domain_name_or_id(self, mock_keystone, mock_get):
        self.op_cloud.update_domain(
            name_or_id='a-domain',
            name='new name',
            description='new description',
            enabled=False)
        mock_get.assert_called_once_with(None, 'a-domain')

    @mock.patch.object(shade.OpenStackCloud, 'keystone_client')
    def test_update_domain_exception(self, mock_keystone):
        mock_keystone.domains.update.side_effect = Exception()
        with testtools.ExpectedException(
            shade.OpenStackCloudException,
            "Error in updating domain domain_id"
        ):
            self.op_cloud.delete_domain('domain_id')



# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

import fixtures
import mock
import os
import os_client_config as occ
from requests import structures
from requests_mock.contrib import fixture as rm_fixture
import tempfile

import shade.openstackcloud
from shade.tests import base


class BaseTestCase(base.TestCase):

    def setUp(self, cloud_config_fixture='clouds.yaml'):
        """Run before each test method to initialize test environment."""

        super(BaseTestCase, self).setUp()

        # Sleeps are for real testing, but unit tests shouldn't need them
        realsleep = time.sleep

        def _nosleep(seconds):
            return realsleep(seconds * 0.0001)

        self.sleep_fixture = self.useFixture(fixtures.MonkeyPatch(
                                             'time.sleep',
                                             _nosleep))
        self.fixtures_directory = 'shade/tests/unit/fixtures'

        # Isolate os-client-config from test environment
        config = tempfile.NamedTemporaryFile(delete=False)
        cloud_path = '%s/clouds/%s' % (self.fixtures_directory,
                                       cloud_config_fixture)
        with open(cloud_path, 'rb') as f:
            content = f.read()
            config.write(content)
        config.close()

        vendor = tempfile.NamedTemporaryFile(delete=False)
        vendor.write(b'{}')
        vendor.close()

        # set record mode depending on environment
        record_mode = os.environ.get('BETAMAX_RECORD_FIXTURES', False)
        if record_mode:
            self.record_fixtures = 'new_episodes'
        else:
            self.record_fixtures = None

        test_cloud = os.environ.get('SHADE_OS_CLOUD', '_test_cloud_')
        self.config = occ.OpenStackConfig(
            config_files=[config.name],
            vendor_files=[vendor.name],
            secure_files=['non-existant'])
        self.cloud_config = self.config.get_one_cloud(
            cloud=test_cloud, validate=False)
        self.cloud = shade.OpenStackCloud(
            cloud_config=self.cloud_config,
            log_inner_exceptions=True)
        self.strict_cloud = shade.OpenStackCloud(
            cloud_config=self.cloud_config,
            log_inner_exceptions=True,
            strict=True)
        self.op_cloud = shade.OperatorCloud(
            cloud_config=self.cloud_config,
            log_inner_exceptions=True)


class TestCase(BaseTestCase):

    def setUp(self, cloud_config_fixture='clouds.yaml'):

        super(TestCase, self).setUp(cloud_config_fixture=cloud_config_fixture)
        self.session_fixture = self.useFixture(fixtures.MonkeyPatch(
            'os_client_config.cloud_config.CloudConfig.get_session',
            mock.Mock()))


class RequestsMockTestCase(BaseTestCase):

    def setUp(self, cloud_config_fixture='clouds.yaml'):

        super(RequestsMockTestCase, self).setUp(
            cloud_config_fixture=cloud_config_fixture)

        self._uri_registry = {}

        self.discovery_json = os.path.join(
            self.fixtures_directory, 'discovery.json')
        self.use_keystone_v3()

    def use_keystone_v3(self):
        self.adapter = self.useFixture(rm_fixture.Fixture())
        self.calls = []
        self.register_uri(
            'GET', 'http://192.168.0.19:35357/',
            text=open(self.discovery_json, 'r').read())
        self.register_uri(
            'POST', 'https://example.com/v3/auth/tokens',
            headers={
                'X-Subject-Token': self.getUniqueString()},
            text=open(
                os.path.join(
                    self.fixtures_directory,
                    'catalog-v3.json'),
                'r').read())
        self._make_test_cloud(identity_api_version='3')

    def use_keystone_v2(self):
        self.adapter = self.useFixture(rm_fixture.Fixture())
        self.calls = []
        self.register_uri(
            'GET', 'http://192.168.0.19:35357/',
            text=open(self.discovery_json, 'r').read())
        self.register_uri(
            'POST', 'https://example.com/v2.0/tokens',
            text=open(
                os.path.join(
                    self.fixtures_directory,
                    'catalog-v2.json'),
                'r').read())
        self._make_test_cloud(identity_api_version='2.0')

    def _make_test_cloud(self, **kwargs):
        test_cloud = os.environ.get('SHADE_OS_CLOUD', '_test_cloud_')
        self.cloud_config = self.config.get_one_cloud(
            cloud=test_cloud, validate=True, **kwargs)
        self.cloud = shade.OpenStackCloud(
            cloud_config=self.cloud_config,
            log_inner_exceptions=True)
        self.op_cloud = shade.OperatorCloud(
            cloud_config=self.cloud_config,
            log_inner_exceptions=True)

    def use_glance(self, image_version_json='image-version.json'):
        discovery_fixture = os.path.join(
            self.fixtures_directory, image_version_json)
        self.register_uri(
            'GET', 'https://image.example.com/',
            text=open(discovery_fixture, 'r').read())

    def register_uri(self, method, uri, **kwargs):
        validate = kwargs.pop('validate', {})
        key = '{method}:{uri}'.format(method=method, uri=uri)
        headers = structures.CaseInsensitiveDict(kwargs.pop('headers', {}))
        if 'content-type' not in headers:
            headers[u'content-type'] = 'application/json'
        kwargs['headers'] = headers

        if key in self._uri_registry:
            self._uri_registry[key].append(kwargs)
            self.adapter.register_uri(method, uri, self._uri_registry[key])
        else:
            self._uri_registry[key] = [kwargs]
            self.adapter.register_uri(method, uri, **kwargs)

        self.calls += [
            dict(
                method=method,
                url=uri, **validate)
        ]

    def assert_calls(self, stop_after=None):
        for (x, (call, history)) in enumerate(
                zip(self.calls, self.adapter.request_history)):
            if stop_after and x > stop_after:
                break
            self.assertEqual(
                call['method'], history.method,
                'Method mismatch on call {index}'.format(index=x))
            self.assertEqual(
                call['url'], history.url,
                'URL mismatch on call {index}'.format(index=x))
            if 'json' in call:
                self.assertEqual(
                    call['json'], history.json(),
                    'json content mismatch in call {index}'.format(index=x))
            # headers in a call isn't exhaustive - it's checking to make sure
            # a specific header or headers are there, not that they are the
            # only headers
            if 'headers' in call:
                for key, value in call['headers'].items():
                    self.assertEqual(
                        value, history.headers[key],
                        'header mismatch in call {index}'.format(index=x))
        self.assertEqual(len(self.calls), len(self.adapter.request_history))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_volume
----------------------------------

Functional tests for `shade` block storage methods.
"""

from testtools import content

from shade.tests.functional import base


class TestVolume(base.BaseFunctionalTestCase):

    def setUp(self):
        super(TestVolume, self).setUp()
        if not self.demo_cloud.has_service('volume'):
            self.skipTest('volume service not supported by cloud')

    def test_volumes(self):
        '''Test volume and snapshot functionality'''
        volume_name = self.getUniqueString()
        snapshot_name = self.getUniqueString()
        self.addDetail('volume', content.text_content(volume_name))
        self.addCleanup(self.cleanup, volume_name, snapshot_name=snapshot_name)
        volume = self.demo_cloud.create_volume(
            display_name=volume_name, size=1)
        snapshot = self.demo_cloud.create_volume_snapshot(
            volume['id'],
            display_name=snapshot_name
        )

        volume_ids = [v['id'] for v in self.demo_cloud.list_volumes()]
        self.assertIn(volume['id'], volume_ids)

        snapshot_list = self.demo_cloud.list_volume_snapshots()
        snapshot_ids = [s['id'] for s in snapshot_list]
        self.assertIn(snapshot['id'], snapshot_ids)

        ret_snapshot = self.demo_cloud.get_volume_snapshot_by_id(
            snapshot['id'])
        self.assertEqual(snapshot['id'], ret_snapshot['id'])

        self.demo_cloud.delete_volume_snapshot(snapshot_name, wait=True)
        self.demo_cloud.delete_volume(volume_name, wait=True)

    def test_volume_to_image(self):
        '''Test volume export to image functionality'''
        volume_name = self.getUniqueString()
        image_name = self.getUniqueString()
        self.addDetail('volume', content.text_content(volume_name))
        self.addCleanup(self.cleanup, volume_name, image_name=image_name)
        volume = self.demo_cloud.create_volume(
            display_name=volume_name, size=1)
        image = self.demo_cloud.create_image(
            image_name, volume=volume, wait=True)

        volume_ids = [v['id'] for v in self.demo_cloud.list_volumes()]
        self.assertIn(volume['id'], volume_ids)

        image_list = self.demo_cloud.list_images()
        image_ids = [s['id'] for s in image_list]
        self.assertIn(image['id'], image_ids)

        self.demo_cloud.delete_image(image_name, wait=True)
        self.demo_cloud.delete_volume(volume_name, wait=True)

    def cleanup(self, volume_name, snapshot_name=None, image_name=None):
        # Need to delete snapshots before volumes
        if snapshot_name:
            snapshot = self.demo_cloud.get_volume_snapshot(snapshot_name)
            if snapshot:
                self.demo_cloud.delete_volume_snapshot(
                    snapshot_name, wait=True)
        if image_name:
            image = self.demo_cloud.get_image(image_name)
            if image:
                self.demo_cloud.delete_image(image_name, wait=True)
        volume = self.demo_cloud.get_volume(volume_name)
        if volume:
            self.demo_cloud.delete_volume(volume_name, wait=True)



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

from shade import task_manager


class UserList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.list()


class UserCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.create(**self.args)


class UserDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.delete(**self.args)


class UserUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.update(**self.args)


class UserPasswordUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.update_password(**self.args)


class UserGet(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.get(**self.args)


class UserAddToGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.add_to_group(**self.args)


class UserCheckInGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.check_in_group(**self.args)


class UserRemoveFromGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.remove_from_group(**self.args)


class ProjectList(task_manager.Task):
    def main(self, client):
        return client._project_manager.list(**self.args)


class ProjectCreate(task_manager.Task):
    def main(self, client):
        return client._project_manager.create(**self.args)


class ProjectDelete(task_manager.Task):
    def main(self, client):
        return client._project_manager.delete(**self.args)


class ProjectUpdate(task_manager.Task):
    def main(self, client):
        return client._project_manager.update(**self.args)


class ServerList(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.list(**self.args)


class ServerListSecurityGroups(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.list_security_group(**self.args)


class ServerConsoleGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.get_console_output(**self.args)


class ServerGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.get(**self.args)


class ServerCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.create(**self.args)


class ServerDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.delete(**self.args)


class ServerUpdate(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.update(**self.args)


class ServerRebuild(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.rebuild(**self.args)


class ServerSetMetadata(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.set_meta(**self.args)


class ServerDeleteMetadata(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.delete_meta(**self.args)


class ServerGroupList(task_manager.Task):
    def main(self, client):
        return client.nova_client.server_groups.list(**self.args)


class ServerGroupGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.server_groups.get(**self.args)


class ServerGroupCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.server_groups.create(**self.args)


class ServerGroupDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.server_groups.delete(**self.args)


class HypervisorList(task_manager.Task):
    def main(self, client):
        return client.nova_client.hypervisors.list(**self.args)


class AggregateList(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.list(**self.args)


class AggregateCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.create(**self.args)


class AggregateUpdate(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.update(**self.args)


class AggregateDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.delete(**self.args)


class AggregateAddHost(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.add_host(**self.args)


class AggregateRemoveHost(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.remove_host(**self.args)


class AggregateSetMetadata(task_manager.Task):
    def main(self, client):
        return client.nova_client.aggregates.set_metadata(**self.args)


class KeypairList(task_manager.Task):
    def main(self, client):
        return client.nova_client.keypairs.list()


class KeypairCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.keypairs.create(**self.args)


class KeypairDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.keypairs.delete(**self.args)


class NetworkList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_networks(**self.args)


class NetworkCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_network(**self.args)


class NetworkDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_network(**self.args)


class RouterList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_routers()


class RouterCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_router(**self.args)


class RouterUpdate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_router(**self.args)


class RouterDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_router(**self.args)


class RouterAddInterface(task_manager.Task):
    def main(self, client):
        return client.neutron_client.add_interface_router(**self.args)


class RouterRemoveInterface(task_manager.Task):
    def main(self, client):
        client.neutron_client.remove_interface_router(**self.args)


class NovaImageList(task_manager.Task):
    def main(self, client):
        return client.nova_client.images.list()


class ImageSnapshotCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.create_image(**self.args)


class VolumeTypeList(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_types.list()


class VolumeTypeAccessList(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_type_access.list(**self.args)


class VolumeTypeAccessAdd(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_type_access.add_project_access(
            **self.args)


class VolumeTypeAccessRemove(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_type_access.remove_project_access(
            **self.args)


class VolumeCreate(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volumes.create(**self.args)


class VolumeDelete(task_manager.Task):
    def main(self, client):
        client.cinder_client.volumes.delete(**self.args)


class VolumeList(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volumes.list()


class VolumeDetach(task_manager.Task):
    def main(self, client):
        client.nova_client.volumes.delete_server_volume(**self.args)


class VolumeAttach(task_manager.Task):
    def main(self, client):
        return client.nova_client.volumes.create_server_volume(**self.args)


class VolumeSnapshotCreate(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_snapshots.create(**self.args)


class VolumeSnapshotGet(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_snapshots.get(**self.args)


class VolumeSnapshotList(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_snapshots.list(**self.args)


class VolumeBackupList(task_manager.Task):
    def main(self, client):
        return client.cinder_client.backups.list(**self.args)


class VolumeBackupCreate(task_manager.Task):
    def main(self, client):
        return client.cinder_client.backups.create(**self.args)


class VolumeBackupDelete(task_manager.Task):
    def main(self, client):
        return client.cinder_client.backups.delete(**self.args)


class VolumeSnapshotDelete(task_manager.Task):
    def main(self, client):
        return client.cinder_client.volume_snapshots.delete(**self.args)


class NeutronSecurityGroupList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_security_groups(**self.args)


class NeutronSecurityGroupCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_security_group(**self.args)


class NeutronSecurityGroupDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_security_group(**self.args)


class NeutronSecurityGroupUpdate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_security_group(**self.args)


class NeutronSecurityGroupRuleCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_security_group_rule(**self.args)


class NeutronSecurityGroupRuleDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_security_group_rule(**self.args)


class NovaSecurityGroupList(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_groups.list(**self.args)


class NovaSecurityGroupCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_groups.create(**self.args)


class NovaSecurityGroupDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_groups.delete(**self.args)


class NovaSecurityGroupUpdate(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_groups.update(**self.args)


class NovaSecurityGroupRuleCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_group_rules.create(**self.args)


class NovaSecurityGroupRuleDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.security_group_rules.delete(**self.args)


class NeutronFloatingIPList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_floatingips(**self.args)


class NovaFloatingIPList(task_manager.Task):
    def main(self, client):
        return client.nova_client.floating_ips.list()


class NeutronFloatingIPCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_floatingip(**self.args)


class NovaFloatingIPCreate(task_manager.Task):
    def main(self, client):
        return client.nova_client.floating_ips.create(**self.args)


class NeutronFloatingIPDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_floatingip(**self.args)


class NovaFloatingIPDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.floating_ips.delete(**self.args)


class NovaFloatingIPAttach(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.add_floating_ip(**self.args)


class NovaFloatingIPDetach(task_manager.Task):
    def main(self, client):
        return client.nova_client.servers.remove_floating_ip(**self.args)


class NeutronFloatingIPUpdate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_floatingip(**self.args)


class FloatingIPPoolList(task_manager.Task):
    def main(self, client):
        return client.nova_client.floating_ip_pools.list()


class SubnetCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_subnet(**self.args)


class SubnetList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_subnets()


class SubnetDelete(task_manager.Task):
    def main(self, client):
        client.neutron_client.delete_subnet(**self.args)


class SubnetUpdate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_subnet(**self.args)


class PortList(task_manager.Task):
    def main(self, client):
        return client.neutron_client.list_ports(**self.args)


class PortCreate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.create_port(**self.args)


class PortUpdate(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_port(**self.args)


class PortDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_port(**self.args)


class MachineCreate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.create(**self.args)


class MachineDelete(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.delete(**self.args)


class MachinePatch(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.update(**self.args)


class MachinePortGet(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.get(**self.args)


class MachinePortGetByAddress(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.get_by_address(**self.args)


class MachinePortCreate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.create(**self.args)


class MachinePortDelete(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.delete(**self.args)


class MachinePortList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.list()


class MachineNodeGet(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.get(**self.args)


class MachineNodeList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.list(**self.args)


class MachineNodePortList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.list_ports(**self.args)


class MachineNodeUpdate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.update(**self.args)


class MachineNodeValidate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.validate(**self.args)


class MachineSetMaintenance(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_maintenance(**self.args)


class MachineSetPower(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_power_state(**self.args)


class MachineSetProvision(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_provision_state(**self.args)


class ServiceCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.create(**self.args)


class ServiceList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.list()


class ServiceUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.update(**self.args)


class ServiceDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.delete(**self.args)


class EndpointCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.create(**self.args)


class EndpointUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.update(**self.args)


class EndpointList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.list()


class EndpointDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.delete(**self.args)


class DomainCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.create(**self.args)


class DomainList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.list(**self.args)


class DomainGet(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.get(**self.args)


class DomainUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.update(**self.args)


class DomainDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.delete(**self.args)


class GroupList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.list()


class GroupCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.create(**self.args)


class GroupDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.delete(**self.args)


class GroupUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.update(**self.args)


class RoleList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.list()


class RoleCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.create(**self.args)


class RoleDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.delete(**self.args)


class RoleAddUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.add_user_role(**self.args)


class RoleGrantUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.grant(**self.args)


class RoleRemoveUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.remove_user_role(**self.args)


class RoleRevokeUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.revoke(**self.args)


class RoleAssignmentList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.role_assignments.list(**self.args)


class RolesForUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.roles_for_user(**self.args)


class StackList(task_manager.Task):
    def main(self, client):
        return client.heat_client.stacks.list()


class StackCreate(task_manager.Task):
    def main(self, client):
        return client.heat_client.stacks.create(**self.args)


class StackUpdate(task_manager.Task):
    def main(self, client):
        return client.heat_client.stacks.update(**self.args)


class StackDelete(task_manager.Task):
    def main(self, client):
        return client.heat_client.stacks.delete(self.args['id'])


class StackGet(task_manager.Task):
    def main(self, client):
        return client.heat_client.stacks.get(**self.args)


class ZoneList(task_manager.Task):
    def main(self, client):
        return client.designate_client.zones.list()


class ZoneCreate(task_manager.Task):
    def main(self, client):
        return client.designate_client.zones.create(**self.args)


class ZoneUpdate(task_manager.Task):
    def main(self, client):
        return client.designate_client.zones.update(**self.args)


class ZoneDelete(task_manager.Task):
    def main(self, client):
        return client.designate_client.zones.delete(**self.args)


class RecordSetList(task_manager.Task):
    def main(self, client):
        return client.designate_client.recordsets.list(**self.args)


class RecordSetGet(task_manager.Task):
    def main(self, client):
        return client.designate_client.recordsets.get(**self.args)


class RecordSetCreate(task_manager.Task):
    def main(self, client):
        return client.designate_client.recordsets.create(**self.args)


class RecordSetUpdate(task_manager.Task):
    def main(self, client):
        return client.designate_client.recordsets.update(**self.args)


class RecordSetDelete(task_manager.Task):
    def main(self, client):
        return client.designate_client.recordsets.delete(**self.args)


class NovaQuotasSet(task_manager.Task):
    def main(self, client):
        return client.nova_client.quotas.update(**self.args)


class NovaQuotasGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.quotas.get(**self.args)


class NovaQuotasDelete(task_manager.Task):
    def main(self, client):
        return client.nova_client.quotas.delete(**self.args)


class NovaUsageGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.usage.get(**self.args)


class CinderQuotasSet(task_manager.Task):
    def main(self, client):
        return client.cinder_client.quotas.update(**self.args)


class CinderQuotasGet(task_manager.Task):
    def main(self, client):
        return client.cinder_client.quotas.get(**self.args)


class CinderQuotasDelete(task_manager.Task):
    def main(self, client):
        return client.cinder_client.quotas.delete(**self.args)


class NeutronQuotasSet(task_manager.Task):
    def main(self, client):
        return client.neutron_client.update_quota(**self.args)


class NeutronQuotasGet(task_manager.Task):
    def main(self, client):
        return client.neutron_client.show_quota(**self.args)['quota']


class NeutronQuotasDelete(task_manager.Task):
    def main(self, client):
        return client.neutron_client.delete_quota(**self.args)


class ClusterTemplateList(task_manager.Task):
    def main(self, client):
        return client.magnum_client.baymodels.list(**self.args)


class ClusterTemplateCreate(task_manager.Task):
    def main(self, client):
        return client.magnum_client.baymodels.create(**self.args)


class ClusterTemplateDelete(task_manager.Task):
    def main(self, client):
        return client.magnum_client.baymodels.delete(self.args['id'])


class ClusterTemplateUpdate(task_manager.Task):
    def main(self, client):
        return client.magnum_client.baymodels.update(
            self.args['id'], self.args['patch'])


class MagnumServicesList(task_manager.Task):
    def main(self, client):
        return client.magnum_client.mservices.list(detail=False)


class NovaLimitsGet(task_manager.Task):
    def main(self, client):
        return client.nova_client.limits.get(**self.args).to_dict()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import shade
from shade.tests import fakes
from shade.tests.unit import base


class TestFlavors(base.RequestsMockTestCase):

    def test_create_flavor(self):

        self.register_uri(
            'POST', '{endpoint}/flavors'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavor': fakes.FAKE_FLAVOR},
            validate=dict(
                json={'flavor': {
                    "name": "vanilla",
                    "ram": 65536,
                    "vcpus": 24,
                    "swap": 0,
                    "os-flavor-access:is_public": True,
                    "rxtx_factor": 1.0,
                    "OS-FLV-EXT-DATA:ephemeral": 0,
                    "disk": 1600,
                    "id": None
                }}))

        self.op_cloud.create_flavor(
            'vanilla', ram=65536, disk=1600, vcpus=24,
        )
        self.assert_calls()

    def test_delete_flavor(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        self.register_uri(
            'DELETE', '{endpoint}/flavors/{id}'.format(
                endpoint=fakes.COMPUTE_ENDPOINT, id=fakes.FLAVOR_ID))
        self.assertTrue(self.op_cloud.delete_flavor('vanilla'))

        self.assert_calls()

    def test_delete_flavor_not_found(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})

        self.assertFalse(self.op_cloud.delete_flavor('invalid'))

        self.assert_calls()

    def test_delete_flavor_exception(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        self.register_uri(
            'DELETE', '{endpoint}/flavors/{id}'.format(
                endpoint=fakes.COMPUTE_ENDPOINT, id=fakes.FLAVOR_ID),
            status_code=503)
        self.assertRaises(shade.OpenStackCloudException,
                          self.op_cloud.delete_flavor, 'vanilla')

    def test_list_flavors(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        for flavor in fakes.FAKE_FLAVOR_LIST:
            self.register_uri(
                'GET', '{endpoint}/flavors/{id}/os-extra_specs'.format(
                    endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                json={'extra_specs': {}})

        flavors = self.cloud.list_flavors()

        # test that new flavor is created correctly
        found = False
        for flavor in flavors:
            if flavor['name'] == 'vanilla':
                found = True
                break
        self.assertTrue(found)
        needed_keys = {'name', 'ram', 'vcpus', 'id', 'is_public', 'disk'}
        if found:
            # check flavor content
            self.assertTrue(needed_keys.issubset(flavor.keys()))
        self.assert_calls()

    def test_get_flavor_by_ram(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        for flavor in fakes.FAKE_FLAVOR_LIST:
            self.register_uri(
                'GET', '{endpoint}/flavors/{id}/os-extra_specs'.format(
                    endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                json={'extra_specs': {}})

        flavor = self.cloud.get_flavor_by_ram(ram=250)
        self.assertEqual(fakes.STRAWBERRY_FLAVOR_ID, flavor['id'])

    def test_get_flavor_by_ram_and_include(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': fakes.FAKE_FLAVOR_LIST})
        for flavor in fakes.FAKE_FLAVOR_LIST:
            self.register_uri(
                'GET', '{endpoint}/flavors/{id}/os-extra_specs'.format(
                    endpoint=fakes.COMPUTE_ENDPOINT, id=flavor['id']),
                json={'extra_specs': {}})
        flavor = self.cloud.get_flavor_by_ram(ram=150, include='strawberry')
        self.assertEqual(fakes.STRAWBERRY_FLAVOR_ID, flavor['id'])

    def test_get_flavor_by_ram_not_found(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': []})
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.get_flavor_by_ram,
            ram=100)

    def test_get_flavor_string_and_int(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': [fakes.make_fake_flavor('1', 'vanilla')]})
        self.register_uri(
            'GET', '{endpoint}/flavors/1/os-extra_specs'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'extra_specs': {}})
        self.register_uri(
            'GET', '{endpoint}/flavors/detail?is_public=None'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'flavors': [fakes.make_fake_flavor('1', 'vanilla')]})
        self.register_uri(
            'GET', '{endpoint}/flavors/1/os-extra_specs'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={'extra_specs': {}})

        flavor1 = self.cloud.get_flavor('1')
        self.assertEqual('1', flavor1['id'])
        flavor2 = self.cloud.get_flavor(1)
        self.assertEqual('1', flavor2['id'])

    def test_set_flavor_specs(self):
        extra_specs = dict(key1='value1')
        self.register_uri(
            'POST', '{endpoint}/flavors/{id}/os-extra_specs'.format(
                endpoint=fakes.COMPUTE_ENDPOINT, id=1),
            json=dict(extra_specs=extra_specs))

        self.op_cloud.set_flavor_specs(1, extra_specs)
        self.assert_calls()

    def test_unset_flavor_specs(self):
        keys = ['key1', 'key2']
        for key in keys:
            self.register_uri(
                'DELETE',
                '{endpoint}/flavors/{id}/os-extra_specs/{key}'.format(
                    endpoint=fakes.COMPUTE_ENDPOINT, id=1, key=key))

        self.op_cloud.unset_flavor_specs(1, keys)
        self.assert_calls()

    def test_add_flavor_access(self):
        self.register_uri(
            'POST', '{endpoint}/flavors/{id}/action'.format(
                endpoint=fakes.COMPUTE_ENDPOINT, id='flavor_id'),
            json={
                'flavor_access': [{
                    'flavor_id': 'flavor_id',
                    'tenant_id': 'tenant_id',
                }]},
            validate=dict(
                json={
                    'addTenantAccess': {
                        'tenant': 'tenant_id',
                    }}))

        self.op_cloud.add_flavor_access('flavor_id', 'tenant_id')
        self.assert_calls()

    def test_remove_flavor_access(self):
        self.register_uri(
            'POST', '{endpoint}/flavors/{id}/action'.format(
                endpoint=fakes.COMPUTE_ENDPOINT, id='flavor_id'),
            json={'flavor_access': []},
            validate=dict(
                json={
                    'removeTenantAccess': {
                        'tenant': 'tenant_id',
                    }}))

        self.op_cloud.remove_flavor_access('flavor_id', 'tenant_id')
        self.assert_calls()

    def test_list_flavor_access(self):
        self.register_uri(
            'GET', '{endpoint}/flavors/vanilla/os-flavor-access'.format(
                endpoint=fakes.COMPUTE_ENDPOINT),
            json={
                'flavor_access': [{
                    'flavor_id': 'vanilla',
                    'tenant_id': 'tenant_id',
                }]})
        self.op_cloud.list_flavor_access('vanilla')
        self.assert_calls()



# Copyright (c) 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

''' Wrapper around keystoneauth Session to wrap calls in TaskManager '''

import functools
from keystoneauth1 import adapter
from six.moves import urllib

from shade import exc
from shade import meta
from shade import task_manager


def extract_name(url):
    '''Produce a key name to use in logging/metrics from the URL path.

    We want to be able to logic/metric sane general things, so we pull
    the url apart to generate names. The function returns a list because
    there are two different ways in which the elements want to be combined
    below (one for logging, one for statsd)

    Some examples are likely useful:

    /servers -> ['servers']
    /servers/{id} -> ['servers']
    /servers/{id}/os-security-groups -> ['servers', 'os-security-groups']
    /v2.0/networks.json -> ['networks']
    '''

    url_path = urllib.parse.urlparse(url).path.strip()
    # Remove / from the beginning to keep the list indexes of interesting
    # things consistent
    if url_path.startswith('/'):
        url_path = url_path[1:]

    # Special case for neutron, which puts .json on the end of urls
    if url_path.endswith('.json'):
        url_path = url_path[:-len('.json')]

    url_parts = url_path.split('/')
    if url_parts[-1] == 'detail':
        # Special case detail calls
        # GET /servers/detail
        # returns ['servers', 'detail']
        name_parts = url_parts[-2:]
    else:
        # Strip leading version piece so that
        # GET /v2.0/networks
        # returns ['networks']
        if url_parts[0] in ('v1', 'v2', 'v2.0'):
            url_parts = url_parts[1:]
        name_parts = []
        # Pull out every other URL portion - so that
        # GET /servers/{id}/os-security-groups
        # returns ['servers', 'os-security-groups']
        for idx in range(0, len(url_parts)):
            if not idx % 2 and url_parts[idx]:
                name_parts.append(url_parts[idx])

    # Keystone Token fetching is a special case, so we name it "tokens"
    if url_path.endswith('tokens'):
        name_parts = ['tokens']

    # Getting the root of an endpoint is doing version discovery
    if not name_parts:
        name_parts = ['discovery']

    # Strip out anything that's empty or None
    return [part for part in name_parts if part]


class ShadeAdapter(adapter.Adapter):

    def __init__(self, shade_logger, manager, *args, **kwargs):
        super(ShadeAdapter, self).__init__(*args, **kwargs)
        self.shade_logger = shade_logger
        self.manager = manager

    def _munch_response(self, response, result_key=None):
        exc.raise_from_response(response)

        if not response.content:
            # This doens't have any content
            return response

        # Some REST calls do not return json content. Don't decode it.
        if 'application/json' not in response.headers.get('Content-Type'):
            return response

        try:
            result_json = response.json()
        except Exception:
            return response

        request_id = response.headers.get('x-openstack-request-id')

        if task_manager._is_listlike(result_json):
            return meta.obj_list_to_dict(
                result_json, request_id=request_id)

        # Wrap the keys() call in list() because in python3 keys returns
        # a "dict_keys" iterator-like object rather than a list
        json_keys = list(result_json.keys())
        if len(json_keys) > 1 and result_key:
            result = result_json[result_key]
        elif len(json_keys) == 1:
            result = result_json[json_keys[0]]
        else:
            # Passthrough the whole body - sometimes (hi glance) things
            # come through without a top-level container. Also, sometimes
            # you need to deal with pagination
            result = result_json

        if task_manager._is_listlike(result):
            return meta.obj_list_to_dict(result, request_id=request_id)
        if task_manager._is_objlike(result):
            return meta.obj_to_dict(result, request_id=request_id)
        return result

    def request(self, url, method, run_async=False, *args, **kwargs):
        name_parts = extract_name(url)
        name = '.'.join([self.service_type, method] + name_parts)
        class_name = "".join([
            part.lower().capitalize() for part in name.split('.')])

        request_method = functools.partial(
            super(ShadeAdapter, self).request, url, method)

        class RequestTask(task_manager.BaseTask):

            def __init__(self, **kw):
                super(RequestTask, self).__init__(**kw)
                self.name = name
                self.__class__.__name__ = str(class_name)
                self.run_async = run_async

            def main(self, client):
                self.args.setdefault('raise_exc', False)
                return request_method(**self.args)

        response = self.manager.submit_task(RequestTask(**kwargs))
        if run_async:
            return response
        else:
            return self._munch_response(response)
